#!/bin/bash

# $Id $

# Nicklas Nordborg

# run ./alignment_statistics.sh path-to-aligned-bam <path-to-unmapped-bam>

ALIGNMENT_BAM=$1
UNMAPPED_BAM=$2

# Flag   Chr	Description
# 0x0001  p	the read is paired in sequencing
# 0x0002  P	the read is mapped in a proper pair
# 0x0004  u	the query sequence itself is unmapped
# 0x0008  U	the mate is unmapped
# 0x0010  r	strand of the query (1 for reverse)
# 0x0020  R	strand of the mate
# 0x0040  1	the read is the first read in a pair
# 0x0080  2	the read is the second read in a pair
# 0x0100  s	the alignment is not primary
# 0x0200  f	the read fails platform/vendor quality checks
# 0x0400  d	the read is either a PCR or an optical duplicate

  SamtoolsVersion=`${SAMTOOLS} |& grep Version`
  echo "${SAMTOOLS} ${SamtoolsVersion}"
  echo "---------------------"
 
if [ -f "${ALIGNMENT_BAM}" ]; then
  cmd="${SAMTOOLS} view -c"
  # skip not primary alignments
  cmd=${cmd}" -F 0x0100"
  # keep first reads and mapped proper pair
  cmd=${cmd}" -f 0x0042" 
  cmd=${cmd}" ${ALIGNMENT_BAM}"
  echo ${cmd}
  ${cmd}

  echo "---------------------"

  cmd="${SAMTOOLS} flagstat"
  cmd=${cmd}" ${ALIGNMENT_BAM}"
  echo ${cmd}
  ${cmd}

  echo "---------------------"
fi

if [ -f "${UNMAPPED_BAM}" ]; then
  cmd="${SAMTOOLS} flagstat"
  cmd=${cmd}" ${UNMAPPED_BAM}"
  echo ${cmd}
 ${cmd}
fi
