#!/bin/sh
# $Id $
#
# Wrapper script to be used when the output to stderr
# should be re-directed to different places depending on
# the exit status of the executed command.
#
# The wrapper calls the program and if the exit status
# is non-zero, stderr is redirected to stderr as usual
# otherwise stderr is redirected to fd3.
# 
# Usage: ./stderrwrap cmd [other parameters] 2> stderr-fail.txt 3> stderr-ok.txt
#

# create temporary file in current directory
TMP=`mktemp --tmpdir=.`

# start program and redirect stderr to the temporary file 
CMD=$1
shift
${CMD} "${@}" 2> ${TMP}

# get exit code
EXIT_CODE=$?

if [ ${EXIT_CODE} -eq 0 ]; then
	# exit code is 0, copy temporary file to fd3
	cat ${TMP} 1>&3
	rm ${TMP}
else
	# exit code is != 0, copy temporary file to stderr
    cat ${TMP} 1>&2
    rm ${TMP}
	# exit with same exit code
	exit ${EXIT_CODE}
fi

