var Hisat = function()
{
	var hisat = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeMergedSequences = null;
	var annotationTypePipeline = null;
	var clusterIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	hisat.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectMergedSequences', hisat.selectMergedSequences);
		Events.addEventHandler('mergedSequences', 'base-selected', hisat.setMergedSequenceCallback);
		Events.addEventHandler('mergedSequences', 'change', hisat.mergedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', hisat.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', hisat.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', hisat.validateStep2);
		Events.addEventHandler('clusters', 'change', hisat.clusterOnChange);
		Events.addEventHandler('maskingSoftware', 'change', hisat.softwareOnChange);
		Events.addEventHandler('alignSoftware', 'change', hisat.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', hisat.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Hisat2023.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMergedSequencesForHisat2023';
		Wizard.asyncJsonRequest(url, hisat.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	hisat.initializeStep1 = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var firstPlate = null;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var libPlate = ms.lib.bioWell ? ms.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = hisat.createListOption(msNo+1, ms, ms.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.mergedSequences.focus();
	}
	
	hisat.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	hisat.selectMergedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.mergedSequences.disabled) return;
		
		if (subtypeMergedSequences == null)
		{
			subtypeMergedSequences = Reggie.getSubtypeInfo('MERGED_SEQUENCES');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeMergedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'=RNAseq';
		Dialogs.selectItem('DERIVEDBIOASSAY', 'mergedSequences', 1, url);
	}

	hisat.setMergedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var opt = Reggie.getListOption('mergedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Hisat2023.servlet?ID='+App.getSessionId();
			url += '&cmd=GetMergedSequencesForHisat2023';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, hisat.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('mergedSequences');
		}
	}

	hisat.manuallySelected = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var offset = frm.mergedSequences.length+1;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var option = hisat.createListOption(msNo+offset, ms, ms.DO_NOT_USE == null);
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
	}
	
	hisat.createListOption = function(index, mergedItem, selected)
	{
		var lib = mergedItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += mergedItem.name;
		
		if (mergedItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(mergedItem.DO_NOT_USE+': '+mergedItem.DO_NOT_USE_COMMENT);
		}
		else if (mergedItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, mergedItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.mergedSequences = mergedItem;
		return option;
	}
	
	hisat.mergedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		
		Wizard.setInputStatus('mergedSequences');
		Wizard.hideGoNextConfirmation();
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				numSelected++;
				var ms = frm.mergedSequences[msNo].mergedSequences;
				if (ms.DO_NOT_USE) numDoNotUse++;
				if (ms.pipeline && ms.pipeline != 'RNAseq')
				{
					invalidPipeline = Strings.encodeTags(ms.name+' ('+ms.pipeline)+') is not intended for the RNAseq pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('mergedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'valid');
		}
	}
	
	// --- Step 2 -----------------------------------
	hisat.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MASKING_PROTOCOL', 'maskingProtocol', 'PIPELINE', 'RNAseq/Hisat/2023');
		Reggie.loadSoftware('MASKING_SOFTWARE', 'maskingSoftware', 'PIPELINE,PARAMETER_SET', 'RNAseq/Hisat/2023');
		Reggie.loadProtocols('ALIGNMENT_PROTOCOL', 'alignProtocol', 'PIPELINE', 'RNAseq/Hisat/2023');
		Reggie.loadSoftware('ALIGNMENT_SOFTWARE', 'alignSoftware', 'PIPELINE,PARAMETER_SET', 'RNAseq/Hisat/2023');
		
		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=align-hisat-2023';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, hisat.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	hisat.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	hisat.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			hisat.clusterOnChange();
		}
	}
	
	hisat.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, hisat.parameterSetInfoLoaded);
		}
	}
	
	hisat.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	hisat.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
		if (cluster.partitions && cluster.partitions.length > 0)
		{
			frm.partition.length = 0;
			for (var pNo = 0; pNo < cluster.partitions.length; pNo++)
			{
				var p = cluster.partitions[pNo];
				var title = p.name;
				if (p.description) title += ' ('+p.description+')';
				frm.partition[frm.partition.length] = new Option(title, p.value, p['default'], p['default']);
			}
			Doc.show('job-partition');
		}
		else
		{
			Doc.hide('job-partition');
		}
	}
	

	hisat.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.maskingSoftware = parseInt(frm.maskingSoftware.value);
		submitInfo.maskingProtocol = parseInt(frm.maskingProtocol.value);
		submitInfo.alignSoftware = parseInt(frm.alignSoftware.value);
		submitInfo.alignProtocol = parseInt(frm.alignProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		if (frm.partition.selectedIndex >= 0)
		{
			submitInfo.partition = frm.partition.value;
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var mergedSequences = [];
		submitInfo.mergedSequences = mergedSequences;
			
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				var ms = {};
				ms.id = frm.mergedSequences[msNo].mergedSequences.id;
				mergedSequences[mergedSequences.length] = ms;
			}
		}

		var url = '../Hisat2023.servlet?ID='+App.getSessionId();
		url += '&cmd=StartHisat2023Alignment';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, hisat.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	hisat.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return hisat;
}();

Doc.onLoad(Hisat.initPage);

