var ViewGenotypes = function()
{
	var vgt = {};
	var debug = 0;
	
	vgt.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetVcfStatistics';
		url += '&fileId='+Data.int('page-data', 'file-id');
		url += '&fileId2='+Data.int('page-data', 'file-id2');
		url += '&itemId='+Data.int('page-data', 'item-id');
		url += '&itemId2='+Data.int('page-data', 'item-id2');
		
		Wizard.showLoadingAnimation('Loading VCF file...');
		Wizard.asyncJsonRequest(url, vgt.onFilesLoaded);
	}
	
	vgt.onFilesLoaded = function(response)
	{
		var alignment1 = response.alignment1 || {};
		var alignment2 = response.alignment2 || {};
		var vcfData1 = response.vcfData1;
		var vcfData2 = response.vcfData2;
		var snpData = response.snpData;
		var compareData = response.compareData;
		var compareMode = compareData != null;
				
		var html = '';
		html += '<tr class="bottomborder topborder">';
		html += '<th>Id</th>';
		html += '<th>Ref</th>';
		html += '<th>Alt</th>';
		html += '<th class="dottedleft"></th>';
		
		html += vgt.createHeaders(vcfData1);
		if (compareMode) 
		{
			html += '<th class="dottedleft"><input type="checkbox" id="hideMatches" title="Show only mismatches"></th>';
			html += vgt.createHeaders(vcfData2);
		}
		
		html += '</tr>';
		for (var snpNo = 0; snpNo < snpData.length; snpNo++)
		{
			var e = snpData[snpNo];
			var snp = e.snp;
			var gt1 = e.gt1;
			var gt2 = e.gt2;
			
			var mismatch = compareMode && gt2 && gt1 && gt1.genoType != gt2.genoType;
			
			html += '<tr class="snp highlight '+(mismatch?'mismatch':'match')+'">';
			html += '<td title="'+Strings.encodeTags(snp.chromosome+':'+snp.position)+'">'+Strings.encodeTags(snp.id)+'</td>';
			html += '<td>'+Strings.encodeTags(snp.ref)+'</td>';
			html += '<td>'+Strings.encodeTags(snp.alt)+'</td>';
			html += vgt.gtToHtml(gt1, vcfData1, 'gt1');
			if (compareMode) html += vgt.gtToHtml(gt2, vcfData2, 'gt2');
			html += '</tr>';
		}
		Doc.element('snp-list').innerHTML = html;

		Doc.element('pipeline').innerHTML = Strings.encodeTags(alignment1.PIPELINE);
		Doc.element('gtCount').innerHTML = vgt.getGtCount(vcfData1, alignment1.QC_GENOTYPE_VERIFIED);
		Doc.element('avgGQ').innerHTML = vgt.getAvgGQ(vcfData1);
		Doc.element('alignedPairs').innerHTML = vgt.getAlignedPairs(alignment1);
		Doc.element('gtComment').innerHTML = Strings.encodeTags(alignment1.QC_GENOTYPE_COMMENT);
		if (compareMode)
		{
			Doc.element('pipeline2').innerHTML = Strings.encodeTags(alignment2.PIPELINE);
			Doc.element('gtCount2').innerHTML = vgt.getGtCount(vcfData2, alignment2.QC_GENOTYPE_VERIFIED);
			Doc.element('avgGQ2').innerHTML = vgt.getAvgGQ(vcfData2);
			Doc.element('alignedPairs2').innerHTML = vgt.getAlignedPairs(alignment2);
			Doc.element('gtComment2').innerHTML = Strings.encodeTags(alignment2.QC_GENOTYPE_COMMENT);
			
			Doc.element('libPlate').innerHTML = Strings.encodeTags(vgt.getLibPlate(alignment1.lib));
			Doc.element('libPlate2').innerHTML = Strings.encodeTags(vgt.getLibPlate(alignment2.lib));

			Doc.element('qiacube').innerHTML = vgt.getQiacube(alignment1);
			Doc.element('qiacube2').innerHTML = vgt.getQiacube(alignment2);

			Doc.element('partitionDate').innerHTML = Reggie.reformatDate(alignment1.PARTITION_DATE);
			Doc.element('partitionDate2').innerHTML = Reggie.reformatDate(alignment2.PARTITION_DATE);

			Doc.element('pat').innerHTML = Strings.encodeTags(alignment1.patientName);
			Doc.element('pat2').innerHTML = Strings.encodeTags(alignment2.patientName);
			
			Doc.element('mismatches-total').innerHTML = compareData.mismatches;
			Doc.element('mismatches-high').innerHTML = compareData.mismatchesHighGQ;
			Doc.element('mismatches-homhom').innerHTML = compareData.homHomMismatches;
			Doc.element('mismatches-hethom').innerHTML = compareData.hetHomMismatches;
			Doc.element('mismatches-homhet').innerHTML = compareData.homHetMismatches;

			Events.addEventHandler('hideMatches', 'change', vgt.showOrHideMatches);
		}
		
		Doc.show('sample-summary', 'table');
	}
	
	
	vgt.createHeaders = function(vcfData)
	{
		var tmp = '<th title="Genotype">GT</th>';
		
		var GQ_header = vcfData.formatHeaders.GQ;
		var GQ_tooltip = 'Genotype Quality';
		if (GQ_header)
		{
			vcfData.gqIsGenCallScore = GQ_header.indexOf('GenCall') > 0;
			GQ_tooltip = GQ_header.replace(/.*Description=\"(.*)\".*/, "$1");
		}		
		
		tmp += '<th title="'+Strings.encodeTags(GQ_tooltip)+'">GQ</th>';
		if (vcfData.formatHeaders.BAF || vcfData.formatHeaders.AF)
		{
			tmp += '<th title="Allele frequency for the alternate allele">AF</th>';
		}
		else
		{
			tmp += '<th title="Allelic depths for the ref and alt alleles in the order listed">AD</th>';
		}
		return tmp;
	}
	
	vgt.gtToHtml = function(gt, vcfData, cls)
	{
		var html = '';
		if (gt)
		{
			if (gt.isLowQuality) cls += ' lowgq';
			html += '<td class="dottedleft '+cls+'">'+Strings.encodeTags(gt.genoType)+'</td>';
			html += '<td class="'+cls+'">'+Strings.encodeTags(gt.GT)+'</td>';
			html += '<td class="'+cls+'">'+(vcfData.gqIsGenCallScore ? Reggie.formatNumber(gt.GQ, '', 2, 2) : gt.GQ)+'</td>';
			html += '<td class="'+cls+'">'+Strings.encodeTags(gt.AD || Reggie.formatNumber(gt.AF, '', 2, 2))+'</td>';
		}
		else
		{
			html += '<td colspan="4" class="dottedleft">-</td>';
		}
		return html;
	}
	
	vgt.getGtCount = function(vcfData, verified)
	{
		var hetPercentage = vcfData.gtCount ? Reggie.formatNumber(100*vcfData.hetCount/vcfData.gtCount, '%', -1) : '-';
		var gtCount = '';
		var icon = '';
		gtCount += vcfData.gtCount;
		gtCount += ' (' + hetPercentage + ' HET';
		if (vcfData.badCount) gtCount += ', ' + vcfData.badCount + ' BAD';
		if (verified)
		{
			gtCount += ', Verified by ' + verified;
			icon = ' <img src="../images/score-complete.png">';
		}
		gtCount += ')'+icon;
		return gtCount;
	}
	
	vgt.getAvgGQ = function(vcfData)
	{
		var avgGq = (vcfData.sumHomGQ+vcfData.sumHetGQ) / vcfData.gtCount;
		var html = '';
		if (vcfData.qualityModel=='HAPLOTYPECALLER')
		{
			html += Numbers.formatNumber(avgGq, 0)
			html += ' (' + vcfData.highGQCount + ' with GQ=99, '+vcfData.lowGQCount + ' with GQ&lt;50)';
		}
		else if (vcfData.qualityModel=='GENCALL')
		{
			html += Numbers.formatNumber(avgGq, 2)
			html += ' (' + vcfData.highGQCount + ' with GQ&gt=0.9, '+vcfData.lowGQCount + ' with GQ&lt;0.2)';
		}
		return html;
	}

	vgt.getLibPlate = function(lib)
	{
		if (!lib) return '';
		var well = lib.bioWell;
		var libPlate = well ? well.bioPlate : null;
		return libPlate ? libPlate.name + ' ' + well.location : '';
	}
	
	vgt.getQiacube = function(alignment)
	{
		if (!alignment || !alignment.QIACUBE_DATE) return '';
		var html = '<span title="Date, #Run number, @Position">'+Reggie.reformatDate(alignment.QIACUBE_DATE);
		html += ', #' + alignment.QIACUBE_RUN_NO;
		html += ', @' + alignment.QIACUBE_POSITION+'</span>';
		return html;
	}
	
	vgt.getAlignedPairs = function(alignment)
	{
		if (!alignment || !alignment.ALIGNED_PAIRS) return '';
		var html = Reggie.formatNumber(alignment.ALIGNED_PAIRS/1000000, 'M', 2);
		if (alignment.FRACTION_DUPLICATION != null)
		{
			html += ' (' + Reggie.formatNumber(100*alignment.FRACTION_DUPLICATION, '%', -1) + ' dup)';
		}
		return html;
	}
	
	vgt.showOrHideMatches = function()
	{
		Doc.addOrRemoveClass('snp-list', 'hidematches');
	}
	
	return vgt;
}();

Doc.onLoad(ViewGenotypes.initPage);
