var TransportBox = function()
{
	var transport = {};
	var debug = 0;
		
	var importData;
	var specimen;
	var boxes;
	
	// Page initialization
	transport.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', transport.validateStep1);

		Wizard.initFileSelectionField('importfile');
		Events.addEventHandler('importfile', 'change', transport.importFileOnChange);
		Events.addEventHandler('sheet', 'change', transport.worksheetOnChange);
		
		Events.addEventHandler('downloadfile', 'click', Files.viewFileOnClick)
		Events.addEventHandler('showSpecimen', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showNoSpecimen', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showCase', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showBlood', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showNothing', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showNoConsent', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showErrors', 'change', transport.toggleItemsToShow);
		Events.addEventHandler('showWarnings', 'change', transport.toggleItemsToShow);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', transport.submit);
	
		Doc.show('step-1');
	}
	
	transport.importFileOnChange = function()
	{
		Doc.hide('worksheetSection');
		Doc.hide('worksheetSection.help');
		// Copy file id to the "Download" icon
		Data.set('downloadfile', 'file-id', Data.get('importfile', 'file-id'));
		transport.prevalidateImportFile();
	}
	
	transport.worksheetOnChange = function()
	{
		transport.prevalidateImportFile();
	}
	
	transport.validateStep1 = function(event)
	{
		if (!Wizard.isValid('importfile'))
		{
			event.preventDefault();
		}
	}

	transport.toggleItemsToShow = function(event)
	{
		Doc.addOrRemoveClass('fileData', Data.get(event.currentTarget, 'toggle'));
	}
	
	transport.resetItemsToShow = function(element, checked)
	{
		element = Doc.element(element);
		element.checked = checked;
		Doc.removeClass('fileData', Data.get(element, 'toggle'));
	}
	
	transport.prevalidateImportFile = function()
	{
		Doc.hide('filePreview');
		Doc.hide('fileData');
		Wizard.hideWizardStatus();
		Wizard.hideGoNextConfirmation();
		transport.resetItemsToShow('showErrors', false);
		transport.resetItemsToShow('showWarnings', false);
		transport.resetItemsToShow('showNoConsent', false);
		transport.resetItemsToShow('showSpecimen', true);
		transport.resetItemsToShow('showNoSpecimen', true);
		transport.resetItemsToShow('showCase', true);
		transport.resetItemsToShow('showBlood', true);
		transport.resetItemsToShow('showNothing', true);
				
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;
		if (path == '') return;
		
		Wizard.setInputStatus('importfile');
		Data.set('importfile', 'valid', 0);
		Doc.hide('gocancel');
		Doc.hide('goregister');
		Doc.removeClass('goregister', 'disabled');

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=PreValidateTransportFile';
		url += '&path='+encodeURIComponent(path);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		Wizard.showLoadingAnimation('Validating ' + Strings.encodeTags(path), 'transport-box-import-progress');
		Wizard.asyncJsonRequest(url, transport.onImportFileValidated, 'POST');
	}

	transport.onImportFileValidated = function(response)
	{
		var frm = document.forms['reggie'];

		Wizard.setInputStatus('importfile');
		Wizard.showFinalMessage(response.messages);
		
		importData = response.importFile;
		specimen = response.specimen;
		boxes = response.boxes;

		var sheetNames = importData.sheetNames;
		if (sheetNames && sheetNames.length)
		{
			var currentSheet = frm.sheet.value;
			frm.sheet.length = 0;
			for (var sNo = 0; sNo < sheetNames.length; sNo++)
			{
				var sheet = sheetNames[sNo];
				frm.sheet[frm.sheet.length] = new Option(sheet, sheet, false, currentSheet == sheet);
			}
			if (frm.sheet.length > 1) 
			{
				Doc.show('worksheetSection');
				Doc.show('worksheetSection.help');
			}
		}
		
		Doc.element('filename').innerHTML = Strings.encodeTags(importData.filename);
		Doc.element('numBoxes').innerHTML = importData.numBoxes;
		Doc.element('numSamples').innerHTML = importData.numSamples;

		Doc.element('numMappedToSpecimen').innerHTML = importData.numMappedToSpecimen;
		Doc.element('numMappedToNoSpecimen').innerHTML = importData.numMappedToNoSpecimen;
		Doc.element('numMappedToCase').innerHTML = importData.numMappedToCase;
		Doc.element('numMappedToBlood').innerHTML = importData.numMappedToBlood;
		Doc.element('numMappedToNothing').innerHTML = importData.numMappedToNothing;
		Doc.element('numNoConsent').innerHTML = importData.numNoConsent;

		Doc.element('numErrors').innerHTML = importData.errors;
		Doc.element('numWarnings').innerHTML = importData.warnings;
		
		Doc.element('importPossible').innerHTML = importData.valid ? '<img src="../images/ok.png">Import is possible!' : '<img src="../images/error.png">Import is not possible!';

		var html = '';
		if (importData.valid)
		{
			for (var boxNo = 0; boxNo < boxes.length; boxNo++)
			{
				var box = boxes[boxNo];
				html += '<div>';
				html += '<b>'+Strings.encodeTags(box.name)+'</b>';
				html += '<span>'+box.numSamples+' samples</span>';
				html += '<span><input type="text" name="box.'+boxNo+'.comments" style="width: 80em;" title="Comments for this box"></span>';
				html += '</div>';
			}
			Doc.element('boxComments').innerHTML = html;
			Doc.show('boxComments');
		}
		else
		{
			Doc.element('boxComments').innerHTML = '';
			Doc.hide('boxComments');
		}
		Doc.show('filePreview');

		var html = '<table id="specimenTable">';
		html += '<thead class="bg-filled-100">';
		// Header row 1
		html += '<tr class="header1">';
		html += '<th></th>';
		html += '<th class="dottedleft" colspan="4">'+Strings.encodeTags(importData.filename)+'</th>';
		html += '<th class="dottedleft" colspan="3">Lorry</th>';
		html += '<th class="dottedleft" colspan="2"></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		// Header row 2
		html += '<tr>';
		html += '<th class="lineNo">Line #</th>';
		html += '<th class="dottedleft">Clarity ID</th>';
		html += '<th>SCAN-B ID</th>';
		html += '<th>PAD</th>';
		html += '<th>Transport box</th>';
		html += '<th class="dottedleft">Storage box</th>'
		html += '<th colspan="2">Mapped item</th>';
		html += '<th class="dottedleft icon"></th>';
		html += '<th>Messages/Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</thead>';
		html += '<tbody>';
		html += '</tr>';
		var lastBox = null;
		for (var sNo = 0; sNo < specimen.length; sNo++)
		{
			var sp = specimen[sNo];
			var well = sp.well;
			var box = well ? well.bioPlate : null;
			var isNewBox = box && lastBox && box.name != lastBox;
			if (box) lastBox = box.name;
			var mappedTo = sp.mappedTo ? sp.mappedTo.itemType : 'Nothing';
			mappedTo += sp.noConsent ? ' NoConsent' : ' HasConsent';
			mappedTo += sp.errors ? ' HasError' : ' NoError';
			mappedTo += sp.warnings ? ' HasWarning' : ' NoWarning';
			
			var icon = 'ok.png';
			if (sp.warnings) icon = 'warning.png';
			if (sp.errors) icon = 'error.png';
			
			html += '<tr class="highlight '+mappedTo+' '+(sNo % 4 < 2 ? 'evenrow' : 'oddrow') + '">';
			html += '<td class="lineNo">'+sp.lineNo+'</td>';
			html += '<td class="dottedleft shaded">';
			if (sp.clarityId)
			{
				html += Strings.encodeTags(sp.clarityId);
				html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(sp.clarityId)+'" title="Show case summary (popup window)">';
			}
			html += '</td>';
			html += '<td class="shaded">';
			if (sp.scanBId)
			{
				html += Strings.encodeTags(sp.scanBId);
				html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(sp.scanBId)+'" title="Show case summary (popup window)">';
			}
			html += '</td>';
			html += '<td class="shaded">';
			if (sp.pad)
			{
				html += Strings.encodeTags(sp.pad);
				html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(sp.pad)+'" title="Show case summary (popup window)">';
			}
			html += '</td>';
			html += '<td class="shaded">'+Strings.encodeTags(sp.transportBox);
			html += '<img src="../images/info.png" class="line-info" title="Line '+sp.lineNo+': '+Strings.encodeTags(sp.line)+'">';
			html += '</td>';
			html += '<td class="dottedleft storage-box '+(isNewBox?'new-box':'')+'">'+(well ? Strings.encodeTags(box.name+' '+well.location):'-')+'</td>';
			html += '<td class="'+(isNewBox?'new-box':'')+'">';
			if (sp.mappedTo)
			{
				html += Strings.encodeTags(sp.mappedTo.name);
				html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(sp.mappedTo.name)+'" title="Show case summary (popup window)">';
			}
			html += '</td>';
			html += '<td class="'+(isNewBox?'new-box':'')+'">';
			if (sp.mappedTo && sp.mappedTo.itemType) 
			{
				html += '<span class="itemsubtype">'+Strings.encodeTags(sp.mappedTo.itemType)+'</span>';
			}
			html += '</td>';
			html += '<td class="dottedleft icon"><img src="../images/'+icon+'"></td>';
			html += '<td>';
			if (sp.alternateAction)
			{
				html += '<div class="action">';
				html += '<label><input type="radio" name="action.'+sNo+'" value="0" '+(sp.alternateIsDefault?'':'checked')+'>';
				html += Strings.encodeTags(sp.action)+'</label></div>';
				html += '<div class="action">';
				html += '<label><input type="radio" name="action.'+sNo+'" value="1" '+(sp.alternateIsDefault?'checked':'')+'>';
				html += Strings.encodeTags(sp.alternateAction)+'</label></div>';
			}
			else
			{
				html += '<div class="action">'+Strings.encodeTags(sp.action)+'</div>';
			}
			var startNo = 1;
			if (sp.errors != null)
			{
				html += '<div class="messagecontainer error"><ol start="'+startNo+'">';
				for (var errNo = 0; errNo < sp.errors.length; errNo++)
				{
					var msg = sp.errors[errNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				if (sp.otherPathNote && !sp.warnings)
				{
					html += '<li><b>Other path. note:</b> '+Strings.encodeTags(sp.otherPathNote);
				}
				
				html += '</ol></div>';
				startNo += sp.errors.length;
			}
			if (sp.warnings != null)
			{
				html += '<div class="messagecontainer note"><ol start="'+startNo+'">';
				for (var warnNo = 0; warnNo < sp.warnings.length; warnNo++)
				{
					var msg = sp.warnings[warnNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				if (sp.otherPathNote)
				{
					html += '<li><b>Other path. note:</b> '+Strings.encodeTags(sp.otherPathNote);
				}
				html += '</ol></div>';
				startNo += sp.warnings.length;
			}

			html += '</td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+sNo+'" value="'+Strings.encodeTags(sp.comment)+'"></td>';
			html += '</tr>';
		}
		html += '</tbody>';
		html += '</table>';
		
		Doc.element('fileData').innerHTML = html;
		if (specimen.length > 0) Doc.show('fileData');
		
		var cs = Doc.element('fileData').getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
		
		Doc.show('gocancel');
		Doc.show('goregister');
		if (importData.valid)
		{
			Wizard.setInputStatus('importfile', importData.warnings > 0 ? 'warning' : 'valid');
			Data.set('importfile', 'valid', 1);
			if (importData.warnings > 0)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration of file with warnings.');
			}
		}
		else
		{
			Wizard.setInputStatus('importfile', 'invalid');
			Doc.addClass('goregister', 'disabled');
		}
	}

	transport.submit = function()
	{
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportFromTransportFile';
		url += '&path='+encodeURIComponent(path);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		var submitInfo = {};
		var boxInfo = {};
		submitInfo.boxes = boxInfo;
		for (var boxNo = 0; boxNo < boxes.length; boxNo++)
		{
			var box = boxes[boxNo];
			var tmp = {};
			tmp.comment = frm['box.'+boxNo+'.comments'].value;
			boxInfo[box.name] = tmp;
		}
		var samples = [];
		submitInfo.samples = samples;
		for (var sNo = 0; sNo < specimen.length; sNo++)
		{
			var sp = specimen[sNo];
			var tmp = {};
			tmp.clarityId = sp.clarityId;
			tmp.comment = frm['comment.'+sNo].value;
			if (frm['action.'+sNo] && frm['action.'+sNo].value == 1)
			{
				tmp.alternateAction = true;
			}
			samples[samples.length] = tmp;
		}
		Wizard.showLoadingAnimation('Performing registration...', 'transport-box-import-progress');
		Wizard.asyncJsonRequest(url, transport.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	transport.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return transport;
}();

Doc.onLoad(TransportBox.initPage);

