var BwaMem2 = function()
{
	var bwa = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	
	var alignedSequences;
	
	// Page initialization
	bwa.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', bwa.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../BwaMem2.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedAlignedSequences';
		Wizard.asyncJsonRequest(url, bwa.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	bwa.initializeStep1 = function(response)
	{
		alignedSequences = response.alignedSequences;
		if (alignedSequences.length == 0)
		{
			Wizard.setFatalError('No aligned sequences available for processing.');
			return;
		}
		var limits = response.limits;
		
		var html = '<table id="alignedSequencesTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft">Sample</th>';
		html += '<th class="dottedleft">PF Reads</th>';
		html += '<th>Aligned pairs</th>';
		html += '<th>Duplicates</th>';
		html += '<th class="dottedleft">PF Bases</th>';
		html += '<th>Q30 Bases</th>';
		html += '<th>Mean</th>';
		html += '<th class="dottedleft" colspan="3">Genotypes QC</th>'
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="2">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">type</th>';
		
		html += '<th class="dottedleft">(millions)</th>';
		html += '<th>(millions)</th>';
		html += '<th>(optical)</th>';
		
		html += '<th class="dottedleft">(billions)</th>';
		html += '<th>(billions)</th>';
		html += '<th>coverage</th>';
		
		html += '<th class="dottedleft">Count</th>';
		html += '<th>HET</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Job</th>';
		html += '<th class="dottedleft">Files</th>';
		html += '<th><span id="realign" data-prefix="realign" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Re-align</span></th>';
		html += '<th><span id="confirmAlign" data-prefix="confirm" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Confirm</span></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		var numWithError = 0;
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var merged = aligned.merged;
			
			var reads = merged.READS == null ? '-' : Numbers.formatNumber(merged.READS/1000000, 1);
			var alignedReads = aligned.ALIGNED_PAIRS == null ? '-' : Numbers.formatNumber(aligned.ALIGNED_PAIRS/1000000, 1);
			var alignedPercent = aligned.ALIGNED_PAIRS && merged.READS ? ' ('+Numbers.formatNumber(100*aligned.ALIGNED_PAIRS/merged.READS, 1, '%')+')' : '';
			var pfBases = aligned.PF_BASES == null ? '-' : Numbers.formatNumber(aligned.PF_BASES/1000000000, 1);
			var q30Bases = aligned.PF_Q30_BASES == null ? '-' : Numbers.formatNumber(aligned.PF_Q30_BASES/1000000000, 1);
			var q30Percent = aligned.PF_Q30_BASES && aligned.PF_BASES ? ' ('+Numbers.formatNumber(100*aligned.PF_Q30_BASES/aligned.PF_BASES, 1, '%')+')' : '';
			var percentDuplicates = aligned.FRACTION_DUPLICATION == null ? '-' : Numbers.formatNumber(100*aligned.FRACTION_DUPLICATION, 1, '%');
			var opticalDuplicates = aligned.FRACTION_OPTICAL_DUPLICATION == null ? '' : ' ('+Numbers.formatNumber(100*aligned.FRACTION_OPTICAL_DUPLICATION, 1, '%')+')';
			var meanCoverage = aligned.MEAN_COVERAGE == null ? '-' : Numbers.formatNumber(aligned.MEAN_COVERAGE, 1);
			var genotypeCount = aligned.QC_GENOTYPE_COUNT || '-';
			var hetPct = aligned.QC_GENOTYPE_HET_PCT == null ? '' : Numbers.formatNumber(aligned.QC_GENOTYPE_HET_PCT, 0, '%');
			var job = aligned.job;

			var isError = job.status == 'ERROR';
			
			// Warnings
			var alignedWarning = '';
			var duplicationWarning = '';
			var hetWarning = '';
			var coverageWarning = '';
			if (!job.debug)
			{
				if (aligned.ALIGNED_PAIRS != null && aligned.ALIGNED_PAIRS < limits.minAlignedPairs)
				{
					alignedWarning = ' <img src="../images/warning.png" title="Less than '+Reggie.formatCount(limits.minAlignedPairs) + ' aligned pairs">';
				}
				if (aligned.FRACTION_DUPLICATION != null && aligned.FRACTION_DUPLICATION > limits.maxFractionDuplication)
				{
					duplicationWarning = ' <img src="../images/warning.png" title="More than '+Math.round(100*limits.maxFractionDuplication)+'% duplication">';
				}
				if (aligned.QC_GENOTYPE_HET_PCT != null && aligned.QC_GENOTYPE_HET_PCT > limits.maxHetPct)
				{
					hetWarning = ' <img src="../images/warning.png" title="More than '+Math.round(limits.maxHetPct)+'% HET may indicate contamination">';
				}
				if (aligned.MEAN_COVERAGE != null && aligned.MEAN_COVERAGE < limits.minMeanCoverage)
				{
					coverageWarning = ' <img src="../images/warning.png" title="Less than '+Math.round(limits.minMeanCoverage)+' coverage">';
				}
			}
			
			html += '<tr class="highlight ' + (alignedNo % 4 < 2 ? 'evenrow' : 'oddrow') + '">';
			html += '<td class="prompt if-yellow">'+aligned.name+'</td>';
			html += '<td class="dottedleft">'+(aligned.PIPELINE=='DNA/Tumor/WGS'?'Tumor':'Normal')+'</td>';
			html += '<td class="dottedleft">'+reads+'</td>';
			html += '<td>'+alignedReads+alignedPercent+alignedWarning+'</td>';
			html += '<td>'+percentDuplicates+opticalDuplicates+duplicationWarning+'</td>';

			html += '<td class="dottedleft">'+pfBases+'</td>';
			html += '<td>'+q30Bases+q30Percent+'</td>';

			html += '<td>'+meanCoverage+coverageWarning+'</td>';
			html += '<td class="dottedleft">'+genotypeCount+'</td>';
			html += '<td>'+hetPct+hetWarning+'</td>';
			html += '<td class="icon-col">';
			if  (aligned.qcVcf)
			{
				html += '<span class="link vcf-link" data-file-id="'+aligned.qcVcf.id+'" data-item-id="'+aligned.id+'" title="View VCF statistics"><img src="../images/vcf_file.png"></span>';
			}
			html += '</td>';
			html += '<td class="dottedleft">';
			html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+job.id+'" title="'+Strings.encodeTags(job.statusMessage)+'"><img src="../images/'+JOB_IMAGE[job.status]+'"></span>';
			if (job.debug)
			{
				html += '<img src="../images/'+(isError?'bug-red':'bug-green')+'.png" title="This job was run in DEBUG mode">'
			}
			html += '</td>';
			if (aligned.DataFilesFolder && job.server)
			{
				html += '<td class="dottedleft">';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(job.server)+'" data-folder="'+Strings.encodeTags(aligned.DataFilesFolder)+'" data-item-id="'+aligned.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft">-</td>';
			}
			
			var confirmChecked = !isError ? ' checked' : '';
			var realignChecked = isError ? ' checked' : '';
			var confirmDisabled = isError ? ' disabled' : '';
			if (isError) numWithError++;

			html += '<td><input type="checkbox" name="realign.'+aligned.id+'"'+realignChecked+'></td>';
			html += '<td><input type="checkbox" name="confirm.'+aligned.id+'"'+confirmChecked+confirmDisabled+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+aligned.id+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('alignedSequences').innerHTML = html;
		if (numWithError > 0) Doc.show('delete-items', 'table');
		
		Events.addEventHandler('realign', 'click', bwa.toggleSelection);
		Events.addEventHandler('confirmAlign', 'click', bwa.toggleSelection);
		
		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', bwa.fileLinkOnClick);
		}

		// Add click handler to VCF files
		var clickableItems = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', bwa.vcfLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	bwa.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = '../analysis/view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&archive=project-archive-dna';
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	bwa.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');

		var url = '../analysis/view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewVcf', 750, 500);
	}
	
	bwa.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var chk = frm[prefix+'.'+aligned.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}
	
	bwa.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.alignedSequences = all;
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var tmp = {};
			tmp.id = aligned.id;
			
			tmp.realign = frm['realign.'+aligned.id].checked;
			tmp.confirm = frm['confirm.'+aligned.id].checked;
			tmp.comment = frm['comment.'+aligned.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../BwaMem2.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterBwaMem2Alignment';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, bwa.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	bwa.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return bwa;
}();

Doc.onLoad(BwaMem2.initPage);

