var BwaMem2 = function()
{
	var bwa = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeMergedSequences = null;
	var annotationTypePipeline = null;
	var clusterIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	bwa.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectMergedSequences', bwa.selectMergedSequences);
		Events.addEventHandler('mergedSequences', 'base-selected', bwa.setMergedSequenceCallback);
		Events.addEventHandler('mergedSequences', 'change', bwa.mergedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', bwa.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', bwa.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', bwa.validateStep2);
		Events.addEventHandler('clusters', 'change', bwa.clusterOnChange);
		Events.addEventHandler('alignSoftware', 'change', bwa.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', bwa.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../BwaMem2.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMergedSequencesForBwaMem2';
		Wizard.asyncJsonRequest(url, bwa.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	bwa.initializeStep1 = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var option = bwa.createListOption(msNo+1, ms, ms.DO_NOT_USE == null)
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.mergedSequences.focus();
	}
	
	bwa.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	bwa.selectMergedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.mergedSequences.disabled) return;
		
		if (subtypeMergedSequences == null)
		{
			subtypeMergedSequences = Reggie.getSubtypeInfo('MERGED_SEQUENCES');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeMergedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent('DNA/Normal/WGS|DNA/Tumor/WGS');
		Dialogs.selectItem('DERIVEDBIOASSAY', 'mergedSequences', 1, url);
	}

	bwa.setMergedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var opt = Reggie.getListOption('mergedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../BwaMem2.servlet?ID='+App.getSessionId();
			url += '&cmd=GetMergedSequencesForBwaMem2';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, bwa.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('mergedSequences');
		}
	}

	bwa.manuallySelected = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var offset = frm.mergedSequences.length+1;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var option = bwa.createListOption(msNo+offset, ms, ms.DO_NOT_USE == null);
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
	}
	
	bwa.createListOption = function(index, mergedItem, selected)
	{
		var tooltip = null;
		var name = (index) + ': '+mergedItem.name;
		if (mergedItem.pipeline) name += ' ('+mergedItem.pipeline+')';
		
		if (mergedItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(mergedItem.DO_NOT_USE+': '+mergedItem.DO_NOT_USE_COMMENT);
		}
		else if (mergedItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, mergedItem.id, false, selected);
		if (tooltip) option.title = tooltip;
		option.mergedSequences = mergedItem;
		return option;
	}
	
	bwa.mergedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		
		Wizard.setInputStatus('mergedSequences');
		Wizard.hideGoNextConfirmation();
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				numSelected++;
				var ms = frm.mergedSequences[msNo].mergedSequences;
				if (ms.DO_NOT_USE) numDoNotUse++;
				if (ms.pipeline && ms.pipeline != 'DNA/Tumor/WGS' && ms.pipeline != 'DNA/Normal/WGS')
				{
					invalidPipeline = Strings.encodeTags(ms.name+' ('+ms.pipeline)+') is not intended for the DNA/*/WGS pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('mergedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'valid');
		}
	}
	
	// --- Step 2 -----------------------------------
	bwa.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('ALIGNMENT_PROTOCOL', 'alignProtocol', 'ALIGNMENT_TYPE,PIPELINE', 'Bwa-mem2');
		Reggie.loadSoftware('ALIGNMENT_SOFTWARE', 'alignSoftware', 'ALIGNMENT_TYPE,PIPELINE,PARAMETER_SET', 'Bwa-mem2');
		
		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=align-bwa-mem2';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, bwa.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	bwa.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	bwa.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			bwa.clusterOnChange();
		}
	}
	
	bwa.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, bwa.parameterSetInfoLoaded);
		}
	}
	
	bwa.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	bwa.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
		if (cluster.partitions && cluster.partitions.length > 0)
		{
			frm.partition.length = 0;
			for (var pNo = 0; pNo < cluster.partitions.length; pNo++)
			{
				var p = cluster.partitions[pNo];
				var title = p.name;
				if (p.description) title += ' ('+p.description+')';
				frm.partition[frm.partition.length] = new Option(title, p.value, p['default'], p['default']);
			}
			Doc.show('job-partition');
		}
		else
		{
			Doc.hide('job-partition');
		}
	}
	

	bwa.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.alignSoftware = parseInt(frm.alignSoftware.value);
		submitInfo.alignProtocol = parseInt(frm.alignProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		if (frm.partition.selectedIndex >= 0)
		{
			submitInfo.partition = frm.partition.value;
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var mergedSequences = [];
		submitInfo.mergedSequences = mergedSequences;
			
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				var ms = {};
				ms.id = frm.mergedSequences[msNo].mergedSequences.id;
				mergedSequences[mergedSequences.length] = ms;
			}
		}

		var url = '../BwaMem2.servlet?ID='+App.getSessionId();
		url += '&cmd=StartBwaMem2Alignment';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, bwa.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	bwa.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return bwa;
}();

Doc.onLoad(BwaMem2.initPage);

