var Pool = function()
{
	var pool = {};
	var debug = 0;

	var plates;
	var nextPoolName;
	var poolsAreValid = false;

	// Page initialization
	pool.initPage = function()
	{
		// Step 1
		Events.addEventHandler('bioplate', 'change', pool.bioplateOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', pool.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', pool.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', pool.validateStep2);
		Events.addEventHandler('poolDate', 'blur', Wizard.validateDate);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', pool.submit);
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLibraryPlatesForPooling';		
		Wizard.showLoadingAnimation('Loading MIPs plates...');
		Wizard.asyncJsonRequest(url, pool.initializeStep1);
	}


	pool.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		nextPoolName = response.nextPoolName;
		Doc.element('poolName').innerHTML = Strings.encodeTags(nextPoolName);
		
		var bioplates = response.bioplates;
		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			var usedBarcodeTemplate = {};
			for (var i=0; i < bioplates.length; i++)
			{
				var plate = bioplates[i];
				var sameTemplate = usedBarcodeTemplate[plate.barcodeTemplate] || false;
				usedBarcodeTemplate[plate.barcodeTemplate] = true;
				var option = new Option(plate.name+' ('+(plate.barcodeTemplate || 'no barcode template')+')', plate.id, !sameTemplate, !sameTemplate);
				option.plate = plate;
				plates.options[plates.length] = option;
			}
			Events.sendChangeEvent('bioplate');
		}
		else
		{
			Wizard.setFatalError('No MIPs plates available for pooling.');
			Doc.hide('gorestart');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	
	// Add pools to the pools list based on the bioplate selection
	pool.bioplateOnChange = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('bioplate');
		poolsAreValid = false;
		
		var numSelected = 0;
		var duplicateBarcodes = false;
		var usedBarcodes = {};
		for (var i = 0; i < frm.bioplate.length; i++)
		{
			if (frm.bioplate[i].selected)
			{
				numSelected++;
				var plate = frm.bioplate[i].plate;
				for (var libNo = 0; libNo < plate.libraries.length; libNo++)
				{
					var barcode = plate.libraries[libNo].barcode;
					if (usedBarcodes[barcode])
					{
						usedBarcodes[barcode]++;
						duplicateBarcodes = true;
					}
					else
					{
						usedBarcodes[barcode] = 1;
					}
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('bioplate', 'invalid', 'Please select at least one plate.');
			return;
		}
		else if (duplicateBarcodes)
		{
			var numDuplicates = 0;
			for (var v in usedBarcodes)
			{
				if (usedBarcodes[v] > 1) numDuplicates += usedBarcodes[v];
			}
			Wizard.setInputStatus('bioplate', 'invalid', 'The selected plates have ' + numDuplicates + ' libraries with duplicate barcode.');
			return;
		}
		
		poolsAreValid = true;
		Wizard.setInputStatus('bioplate', 'valid');
	}
	
	pool.validateStep1 = function(event)
	{
		if (!poolsAreValid) event.preventDefault();
	}
	
	pool.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		Reggie.loadProtocols('POOLING_PROTOCOL', 'poolProtocol', 'LIBPREP_TARGET', 'MIPs');

		frm.poolDate.focus();
	}
	
	pool.validateStep2 = function(event)
	{
		if (!Wizard.isValid('poolDate')) 
		{
			event.preventDefault();
		}
	}
	
	pool.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var plates = [];
		for (var i = 0; i < frm.bioplate.length; i++)
		{
			if (frm.bioplate[i].selected)
			{
				plates[plates.length] = frm.bioplate[i].plate.id;
			}
		}
		
		var submitInfo = {};
		submitInfo.plates = plates;
		submitInfo.poolName = nextPoolName;
		submitInfo.poolDate = frm.poolDate.value;
		submitInfo.poolProtocol = parseInt(frm.poolProtocol.value, 10);
		submitInfo.poolOperator = frm.poolOperator.value;
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterPooledLibraries';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, pool.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	pool.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return pool;
}();

Doc.onLoad(Pool.initPage);

