var Histology = function()
{
	var histology = {};
	var debug = 0;

	var paraffinBlock;
	var wells = ['idx.0.0', 'idx.1.0', 'idx.1.1', 'idx.2.0', 'idx.2.1'];
	var wellIsValid = [];
	var usedWells = [];
	var numHEGlassIsValid = true;
	var existingNumberOfHEGlass = 0;
	var glassBoxIsValid = [];
	
	// Page initialization
	histology.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', histology.validateStep1);
		Buttons.addClickHandler('btnSelectParaffinBlock', histology.selectParaffinBlock);
		Events.addEventHandler('plates', 'base-selected', histology.onParaffinBlockSelected);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', histology.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', histology.validateStep2);
		Events.addEventHandler('stainDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('numberOfHEGlass', 'keypress', Events.integerOnly);
		Events.addEventHandler('numberOfHEGlass', 'change', histology.checkNumHEGlass);
		
		for (var i = 0; i < wells.length; i++)
		{
			var w = wells[i];
			wellIsValid[w] = false;
			Events.addEventHandler(w, 'keypress', Events.integerOnly);
			Events.addEventHandler(w, 'blur', histology.checkIndexValue);
		}
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', histology.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', histology.validateStep3);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', histology.submit);
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetParaffinBlocks';
		Wizard.showLoadingAnimation('Loading paraffin blocks...');
		Wizard.asyncJsonRequest(url, histology.paraffinBlocksLoaded);
	}
	
	histology.paraffinBlocksLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		var plates = response.plates;
		
		var plateList = frm.plates;
		if (plates.length > 0)
		{
			for (var i=0; i < plates.length; i++)
			{
				var plate = plates[i];
				var option = new Option(plate.name, plate.id);
				option.plate = plate;
				plateList.options[plateList.length] = option;
			}
		}
		else
		{	
			Wizard.setInputStatus('plates', 'invalid', 'No paraffin blocks could be found.');
		}

		Doc.show('gonext');
		Doc.show('step-1');
	}


	histology.selectParaffinBlock = function()
	{
		var frm = document.forms['reggie'];
		if (frm.plates.disabled) return;

		var url = '&resetTemporary=1';
		url += '&tmpfilter:STRING:bioPlateType.name=FFPE+block';
		url += '&tmpfilter:INT:freeWells=1';
		Dialogs.selectItem('BIOPLATE', 'plates', 1, url);
	}

	histology.onParaffinBlockSelected = function(event)
	{
		var pb = event.detail;
		Wizard.setInputStatus('plates');
		
		var frm = document.forms['reggie'];
		var plates = frm.plates;
		for (var i = 0; i < plates.length; i++)
		{
			if (plates[i].value == pb.id)
			{
				plates[i].selected = true;
				return;
			}
		}

		var option = new Option(pb.name, pb.id, true, true);
		plates[plates.length] = option;
	}
	
	histology.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		
		if (frm.plates.selectedIndex == -1)
		{
			Wizard.setInputStatus('plates', 'invalid', 'No paraffin block selected.');
			event.preventDefault();
		}
	}
	
	histology.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		Reggie.loadProtocols('STAINING_PROTOCOL', 'stainingProtocol');
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetParaffinBlockInfo&bioPlateId='+frm.plates.value;		
		Wizard.showLoadingAnimation('Loading paraffin block information...');
		Wizard.asyncJsonRequest(url, histology.paraffinBlockInfoLoaded);
	}

	histology.paraffinBlockInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		paraffinBlock = response.paraffinBlock;
		
		var heGlass = paraffinBlock.heGlass;
		var wellMap = [1, 0, 3, 2, 5, 4]; // Map well on Paraffin block to well on HEglass
		
		Doc.element('blockName').innerHTML = Strings.encodeTags(paraffinBlock.name);
		frm.comment.value = paraffinBlock.comment;

		if (heGlass) 
		{
			frm.numberOfHEGlass.value = heGlass.length;
			existingNumberOfHEGlass = heGlass.length;
		}
		else
		{
			existingNumberOfHEGlass = 0;
		}
		
		var numWells = 0;
		for (var i = 0; i < paraffinBlock.bioWells.length; i++)
		{
			var well = paraffinBlock.bioWells[i];
			var div = Doc.element('his.' + well.row + '.' + well.column);
			if (div && well.bioMaterial)
			{
				div.innerHTML = Strings.encodeTags(well.bioMaterial.name);
				Doc.show('input.' + well.row + '.' + well.column);
				usedWells[usedWells.length] = 'idx.' + well.row + '.' + well.column;

				if (heGlass)
				{
					Doc.element('existingGlass').innerHTML = '<img src="../images/info.png">There are ' + heGlass.length + ' HE glass plates already registered for this paraffin block.';
					Doc.show('existingGlass');
					
					// Try to find the current "GoodStain" annotation on a child biomaterial
					var goodStain = 0;
					for (var j = 0; j < heGlass.length; j++)
					{
						var heWell = heGlass[j].bioWells[wellMap[i]];
						if (heWell.bioMaterial && heWell.bioMaterial.GoodStain)
						{
							goodStain = j+1;
							break;
						}
					}
					var field = 'idx.'+well.row + '.' + well.column;
					frm[field].value = goodStain;
					histology.checkIndexValue(field);
				}
				frm['comment.'+well.row+'.'+well.column].value = well.bioMaterial.comment;
				numWells++;
			}
		}
		
		if (numWells == 0)
		{
			Wizard.setFatalError('There are no histology items on this paraffin block.');
			return;
		}
		else if (numWells < 5)
		{
			var emptyLocations = numWells==4 ? '1 empty location' : (5-numWells)+' empty locations';
			Wizard.showGoNextConfirmation(false, 'There is '+emptyLocations+' on this paraffin block.');
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('gonext');
		Doc.show('gocancel');
		frm.stainDate.focus();
	}

	histology.checkNumHEGlass = function()
	{
		var frm = document.forms['reggie'];
		numHEGlassIsValid = false;
		
		var numHEGlass = parseInt(frm.numberOfHEGlass.value, 10) || 0;
		var limit = Math.max(1, existingNumberOfHEGlass);
		if (numHEGlass < limit)
		{
			Wizard.setInputStatus('numberOfHEGlass', 'invalid', 'Must be a value &gt;= ' + limit);
			return;
		}
		
		Wizard.setInputStatus('numberOfHEGlass', 'valid');
		numHEGlassIsValid = true;
		
		for (var i = 0; i < usedWells.length; i++)
		{
			histology.checkIndexValue(usedWells[i]);
		}

	}

	
	histology.checkIndexValue = function(field)
	{
		// Can be called with either a ID or an Event object
		if (field.currentTarget) field = field.currentTarget.id;

		wellIsValid[field] = false;
		
		var frm = document.forms['reggie'];
		var numHEGlass = parseInt(frm.numberOfHEGlass.value, 10) || 1;
		var indexValue = parseInt(frm[field].value, 10);

		if (isNaN(indexValue) || indexValue < 0 || indexValue > numHEGlass)
		{
			Wizard.setInputStatus(field, 'invalid');
			Wizard.setInputStatus('index', '', 'Must be between 1–'+numHEGlass + ' (or 0 if no good stain)');
			return;
		}

		wellIsValid[field] = true;
		if (indexValue == 0)
		{
			Wizard.setInputStatus(field, 'flag', 'No good stain! Sample will be flagged and saved to Flagged Histology list.');
		}
		else
		{
			Wizard.setInputStatus(field, 'valid');
		}

		Wizard.setInputStatus('index');
	}

	
	histology.validateStep2 = function(event)
	{
		var valid = Wizard.isValid('stainDate');
		valid &= numHEGlassIsValid;
		
		for (var i = 0; i < usedWells.length; i++)
		{
			var w = usedWells[i]
			if (!wellIsValid[w])
			{
				Wizard.setInputStatus(w, 'invalid');
				valid = false;
			}
		}
		if (!valid) event.preventDefault();
	}
	
	histology.initializeStep3 = function()
	{
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=FindLastUsedHeGlassPosition';
		Wizard.showLoadingAnimation('Loading HE glass information...');
		Wizard.asyncJsonRequest(url, histology.heGlassInfoLoaded);
	}
	
	histology.heGlassInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		var numHEGlass = parseInt(frm.numberOfHEGlass.value, 10);
		
		// Set flag for slides with at least one good stain item
		var hasGoodStain = [];
		for (var i = 0; i < usedWells.length; i++)
		{
			var w = usedWells[i];
			var glassNo = parseInt(frm[w].value, 10) - 1; 
			hasGoodStain[glassNo] = true;
		}

		var lastHeGlassPosition = response;
		var ihcValues = response.ihc;
		
		var ihcOptions = '<option></option>';
		for (var i = 0; i < ihcValues.length; i++)
		{
			var v = Strings.encodeTags(ihcValues[i]);
			ihcOptions += '<option value="' + v + '">'+v+'</option>';
		}
		
		var html = '<table id="glassTable">';
		html += '<thead>';
		// First header row
		html += '<tr>';
		html += '<th style="width: 10em;">HE glass</th>';
		html += '<th style="width: 8em;">Storage box</th>';
		html += '<th style="width: 8em;">Position</th>';
		html += '<th style="width: 8em;">IHC¹</th>';
		html += '<th style="width: 30em;">Comment</th>';
		html += '<th style="width: 20px;">&nbsp;</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var glassNamePrefix = paraffinBlock.name.replace('PB', 'HE');
		var heGlass = paraffinBlock.heGlass;
		for (var glassNo = 0; glassNo < numHEGlass; glassNo++)
		{
			var heSlide;
			if (heGlass && heGlass[glassNo])
			{
				heSlide = heGlass[glassNo];
			}
			else
			{
				heSlide = {};
				heSlide.name = glassNamePrefix + '.' + (glassNo+1);
			}
			if (!heSlide.position)
			{
				if (lastHeGlassPosition && lastHeGlassPosition.position)
				{
					heSlide.position = ++lastHeGlassPosition.position;
					if (heSlide.position > 100)
					{
						heSlide.position = 1;
						lastHeGlassPosition.position = 1;
						lastHeGlassPosition.storageBox++;
					}
					heSlide.storageBox = lastHeGlassPosition.storageBox;
				}
				else
				{
					heSlide.position = '';
					heSlide.storageBox = '';
				}
			}

			html += '<tr>';
			html += '<td>' + Strings.encodeTags(heSlide.name) + '</td>';
			html += '<td><input type="text" class="required" name="glassBox.'+glassNo+'" id="glassBox.'+glassNo+'" value="'+heSlide.storageBox+'" style="width: 5em;"></td>';
			html += '<td><input type="text" class="required" name="position.'+glassNo+'" id="position.'+glassNo+'" value="'+heSlide.position+'" style="width: 5em;"></td>';
			html += '<td>';
			if (!hasGoodStain[glassNo] || heSlide.IHC)
			{
				html += '<select name="ihc.'+glassNo+'" id="ihc.'+glassNo+'" style="width: 5em;">'+ihcOptions+'</select>';
			}
			html += '</td>';
			html += '<td><input type="text" name="comment.'+glassNo+'"id="comment.'+glassNo+'" style="width: 95%;"></td>';
			html += '<td class="status" id="glassBox.'+glassNo+'.status"></td>';
			html += '</tr>';
			
		}
		html += '</tbody>';
		html += '</table>';
		
		Doc.element('glassBoxInput').innerHTML = html;
		
		for (var glassNo = 0; glassNo < numHEGlass; glassNo++)
		{
			if (heGlass && heGlass[glassNo])
			{
				frm['comment.'+glassNo].value = heGlass[glassNo].comment;
				if (frm['ihc.'+glassNo])
				{
					Forms.selectListOption(frm['ihc.'+glassNo], heGlass[glassNo].IHC);
				}
			}
			
			Events.addEventHandler('glassBox.'+glassNo, 'keypress', Wizard.focusOnEnter, {'next-focus': 'position.'+glassNo});
			Events.addEventHandler('position.'+glassNo, 'keypress', Wizard.focusOnEnter, {'next-focus': 'comment.'+glassNo});
			Events.addEventHandler('comment.'+glassNo, 'keypress', Wizard.focusOnEnter, {'next-focus': 'glassBox.'+(glassNo+1)});
			
			Events.addEventHandler('glassBox.'+glassNo, 'keypress', Events.integerOnly);
			Events.addEventHandler('position.'+glassNo, 'keypress', Events.integerOnly);

			Events.addEventHandler('glassBox.'+glassNo, 'change', histology.glassBoxOnChange, {'index': glassNo});
			Events.addEventHandler('position.'+glassNo, 'change', histology.glassBoxOnChange, {'index': glassNo});
			Events.sendChangeEvent('glassBox.'+glassNo);
		}

		Wizard.setCurrentStep(3);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['glassBox.0'].focus();
	}
	
	histology.glassBoxOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var glassNo = Data.int(event.currentTarget, 'index');
		
		glassBoxIsValid[glassNo] = false;
		
		if (frm['glassBox.'+glassNo].value == '')
		{
			Wizard.setInputStatus('glassBox.'+glassNo, 'invalid', 'Missing storage box');
			return;
		}
		if (frm['position.'+glassNo].value == '')
		{
			Wizard.setInputStatus('glassBox.'+glassNo, 'invalid', 'Missing position');
			return;
		}
		
		glassBoxIsValid[glassNo] = true;
		Wizard.setInputStatus('glassBox.'+glassNo, paraffinBlock.heGlass && paraffinBlock.heGlass[glassNo] ? 'valid' : 'new');
	}

	histology.validateStep3 = function(event)
	{
		var valid = true;
		for (var glassNo = 0; glassNo < glassBoxIsValid.length; glassNo++)
		{
			valid &= glassBoxIsValid[glassNo];
		}
		
		if (!valid) event.preventDefault();
	}
	
	histology.submit = function()
	{
		var frm = document.forms['reggie'];

		for (var i = 0; i < paraffinBlock.bioWells.length; i++)
		{
			var well = paraffinBlock.bioWells[i];
			var field = 'idx.'+ well.row + '.' + well.column;
			if (well.bioMaterial && frm[field])
			{
				well.GoodStain = parseInt(frm[field].value, 10);
				well.bioMaterial.comment = frm['comment.'+well.row+'.'+well.column].value;
			}
		}
		
		if (!paraffinBlock.heGlass) 
		{
			paraffinBlock.heGlass = [];
		}
		
		var glassNo = 0;
		while (frm['glassBox.'+glassNo])
		{
			if (paraffinBlock.heGlass.length <= glassNo)
			{
				paraffinBlock.heGlass[glassNo] = {};
			}
			var heGlass = paraffinBlock.heGlass[glassNo];
			heGlass.storageBox = frm['glassBox.'+glassNo].value;
			heGlass.position = frm['position.'+glassNo].value;
			if (frm['ihc.'+glassNo])
			{
				heGlass.IHC = frm['ihc.'+glassNo].value;
			}
			heGlass.comment = frm['comment.'+glassNo].value;
			glassNo++;
		}
		
		var submitInfo = {};
		paraffinBlock.stainDate = frm.stainDate.value;
		paraffinBlock.protocolId = parseInt(frm.stainingProtocol.value, 10);
		paraffinBlock.comment = frm.comment.value;
		submitInfo.paraffinBlock = paraffinBlock;
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterHEGlass';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, histology.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
		
	histology.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return histology;
}();

Doc.onLoad(Histology.initPage);

