var Protocol = function()
{
	var protocol = {};
	var debug = 0;

	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	

			var boxId = Data.int('page-data', 'transport-box');
			var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
			url += '&cmd=GetTransportBoxDetailsForMove&listId='+boxId;		
			Wizard.showLoadingAnimation('Loading transport box information...');
			Wizard.asyncJsonRequest(url, protocol.initializeProtocol);
		}
		else
		{
			Events.addEventHandler('transportBox', 'change', protocol.transportBoxOnChange);
			Events.addEventHandler('listview', 'click', protocol.viewProtocol);
			Events.addEventHandler('plateview', 'click', protocol.viewProtocol);

			var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
			url += '&cmd=GetTransportBoxesToMove';
			Wizard.showLoadingAnimation('Loading transport boxes...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var transportBoxes = response.transportBoxes;
		var boxes = frm.transportBox;
		if (transportBoxes.length > 0)
		{
			for (var i=0; i < transportBoxes.length; i++)
			{
				var box = transportBoxes[i];
				var name = box.name + ' ('+box.size+' samples)';
				var option = new Option(name, box.id);
				option.box = box;
				boxes.options[boxes.length] = option;
			}
			
			protocol.transportBoxOnChange();
		}
		else
		{
			Wizard.setFatalError('No transport boxes available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	
	// Add pools to the pools list based on the bioplate selection
	protocol.transportBoxOnChange = function()
	{
		var frm = document.forms['reggie'];
		var box = frm.transportBox[frm.transportBox.selectedIndex].box;
		Doc.element('comments').innerHTML = Strings.encodeTags(box.comments);
	}
	
	protocol.viewProtocol = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.transportBox && !frm.transportBox.disabled)
		{
			frm.view.value = Data.get(event.currentTarget, 'protocol-type');
			frm.submit();
		}
	}

	
	protocol.initializeProtocol = function(response)
	{
		var transportBox = response.transportBox;
		var storageBoxes = response.storageBoxes;
		var samples = response.samples;
		
		if (transportBox.externalRef)
		{
			Doc.element('boxName').innerHTML = Strings.encodeTags(transportBox.externalRef);
		}
			
		var boxNames = [];
		for (var boxNo = 0; boxNo < storageBoxes.length; boxNo++)
		{
			var box = storageBoxes[boxNo];
			boxNames[boxNames.length] = Strings.encodeTags(box.name + ' ('+box.freezer+')');
		}
		Doc.element('storageBoxes').innerHTML = boxNames.join('<br>');
		
		var html = '';
		if (transportBox.messages.length > 0)
		{
			html += '<b>Notes</b>';
			html += '<ul>';
			for (var msgNo = 0; msgNo < transportBox.messages.length; msgNo++)
			{
				var msg = transportBox.messages[msgNo];
				// Typically the message starts with a Clarity ID in which case
				// we try to enhance the last digits that are printed larger on the tube
				var ref = msg.match('^(.*?)(\\d+)(\\:.*)$');
				if (ref)
				{
					html += '<li>'+ref[1]+'<span class="major-ref">'+ref[2]+'</span>'+Strings.encodeTags(ref[3]);
				}
				else
				{
					html += '<li>'+Strings.encodeTags(msg);
				}
			}
			html += '</ul>';
			Doc.element('transportMessages').innerHTML = html;
		}
		
		var view = Data.get('page-data', 'view');
		if (view == 'list')
		{
			protocol.viewAsList(samples);
		}
		else
		{
			protocol.viewAsBox(samples)
		}
		Doc.show('all-protocol');
	}
	

	protocol.viewAsBox = function(samples)
	{
		var box1 = new StorageBox('box.1', 9, 9, BoxPainter);
		var box2 = new StorageBox('box.2', 9, 9, BoxPainter);
		
		var currentBox = box1;
		var lastBox = null;
		var addName = true;
		for (var i = 0; i < samples.length; i++)
		{
			var sample = samples[i];
			var well = sample.well;
			if (lastBox != null && well.bioPlate.id != lastBox)
			{
				Doc.show(currentBox.id+'.more-boxes');
				currentBox = box2;
				addName = true;
			}
			if (addName)
			{
				Doc.show(currentBox.id);
				Doc.element(currentBox.id+'.name').innerHTML = well.bioPlate.name;
				lastBox = well.bioPlate.id;
				addName = false;
			}
			Doc.removeClass(currentBox.id+'.row.'+well.row, 'empty-row');
			var tmp = currentBox.getWell(well.row, well.column);
			tmp.setSample(sample);
			Data.set(tmp.tag, 'name', sample.name);
			tmp.tag.title = 'Open case summary: '+sample.name;
			Events.addEventHandler(tmp.tag, 'click', Reggie.openCaseSummaryPopupOnEvent);
		}

		// Normally, the wells before the first one with a sample should have other tubes already
		// we simply "fake" it here by setting the "locked" property
		var wells = box1.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var w = wells[i];
			if (w.sample) break;
			w.setLocked(true);
		}

		box1.paint(box1.getWells());
		box2.paint(box2.getWells());
	}

	protocol.viewAsList = function(samples)
	{
		var html = '';
		var lastRow = -1;
		var rowClass = 'odd-row';
		for (var i = 0; i < samples.length; i++)
		{
			var sample = samples[i];
			var well = sample.well;
			if (lastRow != well.row)
			{
				rowClass = rowClass=='even-row' ? 'odd-row' : 'even-row';
			}
			lastRow = well.row;
			
			html += '<tr class="'+rowClass+'">';
			html += '<td class="clarityId">';
			// We try to enhance the last digits that are printed larger on the tube
			// Eg. cmd:CMD655A21 --> CMD655A (21)
			var ref = sample.externalRef.match('^cmd\\:(.*?)(\\d+)$');
			if (ref) 
			{
				html += Strings.encodeTags(ref[1])+'<span class="major-ref">'+Strings.encodeTags(ref[2])+'</span>';
			}
			else
			{
				html += Strings.encodeTags(sample.externalRef);
			}
			html += '</td>';
			html += '<td class="specimen">'+Strings.encodeTags(sample.specimenName || sample.name)+'</td>';
			html += '<td class="box">'+Strings.encodeTags(well.bioPlate.name)+'</td>';
			html += '<td class="location">'+Strings.encodeTags(well.location)+'</td>';
			html += '<td class="remain">'+Protocol.formatRemainingQuantity(sample.remain, sample.type)+'</td>';
			html += '<td class="comment">'+Strings.encodeTags(sample.comment)+'</td>';
			html += '</tr>';
		}
		Doc.element('listData').innerHTML = html;
	}

	protocol.formatRemainingQuantity = function(remain, sampleType)
	{
		var result = '';
		if (sampleType == 'Specimen')
		{
			result = Numbers.formatNumber(remain/1000, 0, ' mg');
		}
		else if (sampleType == 'Lysate' || sampleType == 'FlowThrough')
		{
			result = Numbers.formatNumber(remain, 0, ' µl');
		}
		else // RNA and DNA
		{
			result = Numbers.formatNumber(remain, 1, ' µg');
		}
		return result;
	}
	
	return protocol;
}();

Doc.onLoad(Protocol.initPage);


var BoxPainter = function()
{
	var painter = {};
	
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (well.sample)
		{
			cls += ' not-empty';
		}
		if (well.locked)
		{
			cls += ' locked';
		}
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		var sample = well.sample;
		if (sample)
		{
			// We try to enhance the last digits that are printed larger on the tube
			// Eg. cmd:CMD655A21 --> CMD655A (21)
			var ref = sample.externalRef.match('^cmd\\:(.*?)(\\d+)$');
			if (ref)
			{
				text += '<div class="minor-ref">'+Strings.encodeTags(ref[1])+'</div>';
				text += '<div class="major-ref">'+Strings.encodeTags(ref[2])+'</div>';
			}
			else
			{
				text += '<div class="major-ref">'+Strings.encodeTags(sample.externalRef)+'</div>';
			}
			text += '<div class="specimen-name">'+Strings.encodeTags(sample.specimenName || sample.name)+'</div>';
			if (sample.remain != null)
			{
				text += '<div class="remain">';
				text += Protocol.formatRemainingQuantity(sample.remain, sample.type);
				text += '</div>';
			}
		}
		else if (well.locked)
		{
			text = '×';
		}
		return text;
	}

	return painter;
}();

