var Move = function()
{
	var move = {};
	var debug = 0;

	var samples;
	var prefixes;

	var missingSamplesIsValid = true;
	var extraSamplesIsValid = true;
	

	// Page initialization
	move.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', move.validateStep1);
		Events.addEventHandler('transportBox', 'change', move.transportBoxOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', move.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', move.validateStep2);
		Events.addEventHandler('moveDate', 'change', Wizard.validateDate);
		Events.addEventHandler('noMissingSamples', 'click', move.noMissingSamplesOnClick);
		Events.addEventHandler('missingSamples', 'change', move.missingSamplesOnChange);
		Events.addEventHandler('extraSamples', 'change', move.extraSamplesOnChange);
		Buttons.addClickHandler('showMissingSampleMenu', move.showMissingSamplesMenu);
		Events.addEventHandler('select-missing-sample', 'click', move.missingSampleSelected);
		Events.addEventHandler(document.body, 'click', move.hideMissingSampleMenu);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', move.submit);

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetTransportBoxesToMove';
		Wizard.showLoadingAnimation('Loading transport boxes...');
		Wizard.asyncJsonRequest(url, move.initializeStep1);
	}

	move.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var transportBoxes = response.transportBoxes;
		var boxes = frm.transportBox;
		if (transportBoxes.length > 0)
		{
			for (var i=0; i < transportBoxes.length; i++)
			{
				var box = transportBoxes[i];
				var name = box.name + ' ('+box.size+' samples)';
				var option = new Option(name, box.id);
				option.box = box;
				boxes.options[boxes.length] = option;
			}
			
			move.transportBoxOnChange();
		}
		else
		{
			Wizard.setFatalError('No transport boxes available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	move.transportBoxOnChange = function()
	{
		var frm = document.forms['reggie'];
		var box = frm.transportBox[frm.transportBox.selectedIndex].box;
		Doc.element('boxComments').innerHTML = Strings.encodeTags(box.comments);
		frm.comments.value = box.comments;
	}
	
	move.validateStep1 = function(event)
	{}
	
	move.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetTransportBoxDetailsForMove&listId='+frm.transportBox.value;		
		Wizard.showLoadingAnimation('Loading transport box information...');
		Wizard.asyncJsonRequest(url, move.transportBoxLoaded);
	}
	
	move.transportBoxLoaded = function(response)
	{
		samples = response.samples;
		prefixes = response.prefixes;
		
		var selectMissingSample = '';
		var lastBoxRow = 0;
		for (var i = 0; i < samples.length; i++)
		{
			var sample = samples[i];
			var ref = sample.clarityId.match('^(.*?)(\\d+)$');
			if (i > 0 && lastBoxRow != sample.well.row)
			{
				lastBoxRow = sample.well.row;
				selectMissingSample += '<div class="menuseparator"></div>';
			}
			selectMissingSample += '<div class="menuitem enabled interactable" id="sample-'+sample.id+'" data-clarity-id="'+sample.clarityId+'">';
			selectMissingSample += Strings.encodeTags(sample.well.location+': ');
			if (ref)
			{
				selectMissingSample += ref[1]+'<b>'+ref[2]+'</b>';
			}
			else
			{
				selectMissingSample += Strings.encodeTags(sample.clarityId);
			}
			selectMissingSample += ' - '+Strings.encodeTags(sample.specimenName);
			selectMissingSample += '</div>';
		}
		Doc.element('missing-samples-list').innerHTML = selectMissingSample;

		Wizard.setCurrentStep(2);
		Doc.show('step-2');
		Doc.show('gocancel');
		Doc.show('goregister');
		Doc.element('moveDate').focus();
	}
	
	move.noMissingSamplesOnClick = function()
	{
		var frm = document.forms['reggie'];
		
		if (frm.noMissingSamples.checked)
		{
			frm.missingSamples.storedValue = frm.missingSamples.value;
			frm.missingSamples.value = '';
			Wizard.setInputStatus('missingSamples', 'valid');
		}
		else
		{
			frm.missingSamples.value = frm.missingSamples.storedValue || '';
			frm.missingSamples.focus();
			if (frm.missingSamples.value != '') Events.sendChangeEvent('missingSamples');
		}	
	}
	
	move.missingSamplesOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		Wizard.setInputStatus('missingSamples');
		missingSamplesIsValid = true;
		var missing = Strings.trim(frm.missingSamples.value).toUpperCase();
		
		frm.noMissingSamples.checked = missing == '';
		if (missing != '')
		{
			var invalid = [];
			var ids = missing.match(/\w+/g);
			for (var i=0; i < ids.length; i++)
			{
				var sample = move.getSampleWithClarityId(ids[i]);
				if (!sample) invalid[invalid.length]=ids[i];
			}
			if (invalid.length > 0)
			{
				missingSamplesIsValid = false;
				Wizard.setInputStatus('missingSamples', 'invalid', 'Sample(s) "'+invalid.join(', ')+'" was not expected in this box');
			}
			else
			{
				Wizard.setInputStatus('missingSamples', 'valid');
			}
		}	
	}
	
	
	move.showMissingSamplesMenu = function(event)
	{
		var frm = document.forms['reggie'];
		// Mark items as selected in the menu
		var missing = Strings.trim(frm.missingSamples.value).toUpperCase();
		var ids = missing.match(/\w+/g) || [];
		for (var i = 0; i < samples.length; i++)
		{
			var sample = samples[i];
			Doc.addOrRemoveClass('sample-'+sample.id, 'is-selected', ids.indexOf(sample.clarityId)>=0);
		}
		var menu = Doc.element('select-missing-sample');
		var x = event.clientX;
		var y = event.clientY+2;
		menu.style.display = 'block';
		menu.style.left = (x)+'px';
		menu.style.top = (y)+'px';
		event.stopPropagation();
	}
	
	move.missingSampleSelected = function(event)
	{
		var target = event.target;
		if (target.tagName != 'DIV') target = target.parentNode;
		var frm = document.forms['reggie'];
		var clarityId = Data.get(target, 'clarity-id');
		if (!clarityId) return;
		
		var missing = frm.missingSamples.value;
		var found = missing.indexOf(clarityId);
		if (found >= 0)
		{
			missing = missing.substring(0, found)+missing.substring(found+clarityId.length);
			missing = missing.replace(/\s\s/g, ' ').trim();
		}
		else
		{
			if (missing.length > 0 && missing.charAt(missing.length-1)!=' ')
			{
				missing+=' ';
			}
			missing += clarityId;	
		}
		frm.missingSamples.value = missing;
		Events.sendChangeEvent('missingSamples');
	}
	
	move.hideMissingSampleMenu = function()
	{
		Doc.hide('select-missing-sample');
	}
	
	move.extraSamplesOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		Wizard.setInputStatus('extraSamples');
		extraSamplesIsValid = true;
		var extra = Strings.trim(frm.extraSamples.value).toUpperCase();
		
		if (extra != '')
		{
			var noMatchingPrefix = [];
			var matchedSamples = [];
			var ids = extra.match(/\w+/g);
			for (var i=0; i < ids.length; i++)
			{
				var id = ids[i];
				if (move.getSampleWithClarityId(id))
				{
					matchedSamples[matchedSamples.length]=id;
				}
				else if (!move.isMatchingPrefix(id))
				{
					noMatchingPrefix[noMatchingPrefix.length] = id;
				}
			}
			if (matchedSamples.length > 0)
			{
				extraSamplesIsValid = false;
				Wizard.setInputStatus('extraSamples', 'invalid', 'Sample(s) "'+matchedSamples.join(', ')+'" are expected in this transport box');
			}
			else if (noMatchingPrefix.length > 0)
			{
				Wizard.setInputStatus('extraSamples', 'warning', 'Sample(s) "'+noMatchingPrefix.join(', ')+'" has unexpected prefix. Expected one of: '+prefixes.join(', '));
			}
			else
			{
				Wizard.setInputStatus('extraSamples', 'valid');
			}
		}
		
	}
	
	move.getSampleWithClarityId = function(clarityId)
	{
		for (var sampleNo = 0; sampleNo < samples.length; sampleNo++)
		{
			var s = samples[sampleNo];
			if (s.clarityId == clarityId) return s;
		}
		return null;
	}
	
	move.isMatchingPrefix = function(clarityId)
	{
		for (var prefixNo = 0; prefixNo < prefixes.length; prefixNo++)
		{
			if (clarityId.indexOf(prefixes[prefixNo]) == 0) return true;
		}
		return false;
	}
	
	move.validateStep2 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('moveDate');
		valid &= missingSamplesIsValid;
		valid &= extraSamplesIsValid;
		if (!valid) event.preventDefault();
	}
	
	move.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		
		submitInfo.transportBox = parseInt(frm.transportBox.value);
		submitInfo.moveDate = frm.moveDate.value;
		submitInfo.moveOperator = frm.moveOperator.value;
		submitInfo.extraSamples = frm.extraSamples.value;
		submitInfo.comments = frm.comments.value;
		
		var missingSamples = [];
		submitInfo.missingSamples = missingSamples;
		
		var missing = Strings.trim(frm.missingSamples.value).toUpperCase();
		if (missing != '')
		{
			var ids = missing.match(/\w+/g);
			for (var i=0; i < ids.length; i++)
			{
				var clarityId = ids[i];
				var sample = move.getSampleWithClarityId(clarityId);
				var tmp = {};
				tmp.id = sample.id;
				tmp.clarityId = clarityId;
				missingSamples[missingSamples.length] = tmp;
			}
		}
		
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterMoveComplete';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, move.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	move.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return move;
}();

Doc.onLoad(Move.initPage);
