// Inpsired from plate.js, but with support for multiple boxes on
// one page but otherwise a lot simpler
function StorageBox(boxId, rows, columns, painter)
{
	this.id = boxId;
	this.rows = rows;
	this.columns = columns;
	this.painter = painter;
	this.tag = document.getElementById(boxId);
	
	this.wells = [];
	// Wells are ordered by row instead of by column
	for (var r = 0; r < rows; r++)
	{
		for (var c = 0; c < columns; c++)
		{
			this.wells[this.wells.length] = new Well(this, r, c);
		}
	}
}
	
	
/**
	Get the number of rows in the storage box.
*/
StorageBox.prototype.getRows = function()
{
	return this.rows;
}

/**
	Get the number of columns in the storage box.
*/
StorageBox.prototype.getColumns = function()
{
	return this.columns;
}

	
/**
	Get all wells in the storage box.
*/
StorageBox.prototype.getWells = function()
{
	return this.wells;
}

	
/**
	Get the well at the given coordinate.
*/
StorageBox.prototype.getWell = function(row, column)
{
	return this.wells[row * this.columns + column];
}
	
	
/**
	Paint the given wells.
*/
StorageBox.prototype.paint = function(wells)
{
	for (var i = 0; i < wells.length; i++)
	{
		wells[i].paint(this.painter);
	}
}
	

/**
	Represents a well on the plate. Each well is created when 
	the page is loaded and is initially empty.
*/
function Well(box, row, column)
{
	this.box = box;
	this.row = row;
	this.column = column;
	this.selected = false;
	this.highlighClass = null;
	this.sample = null;
	this.locked = false;
	
	this.tag = document.getElementById(box.id+'.well.'+row+'.'+column);
}

/**
	Set the samples item that is put into this well. Use
	'null' to clear the well. The sample should always
	have a 'name' property, and an 'id' property if it 
	exists in the database.
*/
Well.prototype.setSample = function(sample)
{
	this.sample = sample;
}

Well.prototype.setLocked = function(locked)
{
	this.locked = locked;
}

Well.prototype.getClassName = function(painter)
{
	var c = this.column;
	var cls = 'well col-'+c;
	
	if (painter)
	{
		cls += ' ' + painter.getClassNameForWell(this);
	}
	return cls;
}

Well.prototype.getText = function(painter)
{
	var text = '';
	if (painter)
	{
		text = painter.getWellText(this);
	}
	else
	{
		if (this.sample)
		{
			text += '<div class="name">'+Strings.encodeTags(this.sample.name)+'</div>'
		}
	}
	return text;
}

Well.prototype.paint = function(painter)
{
	this.tag.innerHTML = this.getText(painter);
	this.tag.className = this.getClassName(painter);
}
