var YellowLabel = function()
{
	var yellow = {};
	var debug = 0;
	var home;
	
	var specimen;
	var sites;
	var permissions;
	
	// Page initialization
	yellow.initPage = function()
	{
		home = Data.get('page-data', 'home-reggie');
		
		// Step 1
		Events.addEventHandler('nameFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('siteFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('samplingDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('arrivalDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('consentDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('extractionDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('rnaqcFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('libraryDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('rawBioAssayDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('pilotReportDateFilter', 'change', yellow.filterOnChange);
		Events.addEventHandler('completedDateFilter', 'change', yellow.completedDateFilterOnChange);
		Events.addEventHandler('downloadedDateFilter', 'change', yellow.filterOnChange);

		Buttons.addClickHandler('check_uncheck', yellow.selectAll);
		Buttons.addClickHandler('btnMarkAsCompleted', yellow.markAsCompleted);
		Buttons.addClickHandler('btnCreatePilotReportZip', yellow.createPilotReportZip);
		Buttons.addClickHandler('btnCreatePilotReportPdf', yellow.createPilotReportPdf);
		Buttons.addClickHandler('btnPublishForDelivery', yellow.publishForDelivery);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', yellow.submit);

		var url = '../YellowLabel.servlet?ID='+App.getSessionId();
		url += '&cmd=LoadSitesAndPermissions';
		Wizard.asyncJsonRequest(url, yellow.onSitesLoaded);
		Wizard.showLoadingAnimation('Loading sites...');
	}
	
	yellow.onSitesLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		sites = response.sites;
		permissions = response.permissions;
		if (permissions.PatientCurator || permissions.Administrator)
		{
			Doc.show('yellow-toolbar');
		}
		
		for (var siteNo = 0; siteNo < sites.length; siteNo++)
		{
			var site = sites[siteNo];
			frm.siteFilter[frm.siteFilter.length] = new Option(site.name, site.prefix);
		}
		
		var url = '../YellowLabel.servlet?ID='+App.getSessionId();
		url += '&cmd=LoadActiveYellowSpecimen';
		Wizard.showLoadingAnimation('Loading YellowLabel specimen...');
		Wizard.asyncJsonRequest(url, yellow.onSpecimenLoaded);
	}
	
	yellow.onSpecimenLoaded = function(response)
	{
		specimen = response.specimen;
		var frm = document.forms['reggie'];
		
		var siteCount = [];
		for (var siteNo = 0; siteNo < sites.length; siteNo++)
		{
			siteCount[sites[siteNo].prefix] = 0;
		}

		var html = '';
		var numActive = 0;
		var numCompleted = 0;
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			var rna = sp.rna;
			var rnaqc = sp.rnaqc;
			var lib = sp.library;
			var raw = sp.raw;
			var pilotReport = sp.pilotReport;
			
			sp.include = true;
			var rowClass = 'highlight';
			rowClass += spNo % 2 == 0 ? ' oddrow' : ' evenrow';
			if (sp.CompletedDate) 
			{
				rowClass += ' completed';
			}
			else if (sp.pilotReport)
			{
				rowClass += ' pilotreport';
			}
			if (raw && raw.DO_NOT_USE || rna && rna.DO_NOT_USE)
			{
				rowClass += ' donotuse';
			}
			
			html += '<tr class="' + rowClass + '" id="sp.'+sp.id + '">';
			html += '<td>';
			siteCount[sp.site.prefix]++;
			if (!sp.CompletedDate)
			{
				numActive++;
				html += '<span id="index.'+sp.id+'">'+numActive+'</span> <input type="checkbox" name="sp.'+sp.id+'">';
			}
			else
			{
				numCompleted++;
			}
			html += '</td>';
			html += '<td class="dottedleft"><span class="link linked-item" data-item-type="SAMPLE" data-item-id="'+sp.id+'">'+Strings.encodeTags(sp.name)+'</span></td>';
			html += '<td class="iconcol-2"><img class="link case-summary" data-name="'+Strings.encodeTags(sp.name)+'" src="../images/case_summary.png">'+yellow.doNotUseWarning(raw, rna)+'</td>';
			html += '<td>'+Strings.encodeTags(sp.site.name) + '</td>';
			html += '<td>'+Reggie.reformatDate(sp.SamplingDate)+'</td>';
			html += '<td>'+Reggie.reformatDate(sp.ArrivalDate)+'</td>';
			html += '<td>'+Reggie.reformatDate(sp.ConsentDate, '<span class="missing">Missing</span>')+'</td>';
			html += '<td class="dottedleft">'+(rna ? Reggie.reformatDate(rna.QiacubeDate, '<span class="in-progress">In progress</span>'):'')+'</td>';
			html += '<td class="iconcol-2">'+yellow.asItemLink(rna, 'EXTRACT')+yellow.asFlag(rna)+'</td>';
			html += '<td>'+(rnaqc ? Reggie.formatNumber(rnaqc.RQS || rnaqc.RIN, '', 1) : (rna && rna.QiacubeDate ? '<span class="in-progress">In progress</span>' : ''))+'</td>';
			html += '<td class="dottedleft">'+(lib ? Reggie.reformatDate(lib.creationDate, '<span class="in-progress">In progress</span>'):'')+'</td>';
			html += '<td class="iconcol-1">'+yellow.asItemLink(lib, 'EXTRACT')+'</td>';
			html += '<td class="dottedleft">'+(raw ? Reggie.reformatDate(raw.creationDate, '<span class="in-progress">In progress</span>'):'')+'</td>';
			html += '<td class="iconcol-1">'+yellow.asItemLink(raw, 'RAWBIOASSAY')+'</td>';
			html += '<td class="dottedleft">'+(pilotReport ? Reggie.reformatDate(pilotReport.lastUpdatedDate):'')+'</td>';
			html += '<td class="iconcol-1">'+yellow.asFileLink(pilotReport, 'pdffile.png')+'</td>';
			html += '<td class="dottedleft">'+Reggie.reformatDate(sp.CompletedDate)+'</td>';
			html += '<td class="dottedleft">';
			if (pilotReport)
			{
				if (pilotReport.isPublished)
				{
					if (pilotReport.downloadCount > 0)
					{
						html += Reggie.reformatDate(pilotReport.downloadedDate)+' ('+pilotReport.downloadCount+')';
					}
					else
					{
						html += 'Not downloaded';
					}
				}
				else if (sp.CompletedDate)
				{
					html += 'Not published';
				}
			}
			html += '</td>';
			html += '</tr>';
		}
		
		// Update the site filter list with current counts
		for (var siteNo = 0; siteNo < sites.length; siteNo++)
		{
			var site = sites[siteNo];
			frm.siteFilter[siteNo+1].text = site.name + ' ('+siteCount[site.prefix]+')'; 
		}
		
		Doc.element('specimen-list').innerHTML = html;
		Doc.element('num-active').innerHTML = numActive;
		Doc.element('num-completed').innerHTML = numCompleted;
		
		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
		
		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}

		var linkedFiles = document.getElementsByClassName('linked-file');
		for (var fileNo = 0; fileNo < linkedFiles.length; fileNo++)
		{
			Events.addEventHandler(linkedFiles[fileNo], 'click', Files.viewFileOnClick);
		}
		
		Doc.show('step-1');
		
		yellow.filterOnChange();

	}
	
	yellow.selectAll = function()
	{
		var frm = document.forms['reggie'];
		var firstChecked = null;
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			var checkbox = frm['sp.'+sp.id];
			if (sp.include && checkbox)
			{
				if (firstChecked == null)
				{
					firstChecked = checkbox.checked;
				}
				checkbox.checked = !firstChecked;
			}
		}
	}
	
	yellow.completedDateFilterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var url = '../YellowLabel.servlet?ID='+App.getSessionId();
		var completedDate = frm.completedDateFilter.value.replace(/\-/g, '');
		Doc.element('specimen-list').innerHTML = '';
		url += '&cmd=LoadActiveYellowSpecimen&completedDateFilter='+encodeURIComponent(completedDate);
		Wizard.showLoadingAnimation('Loading YellowLabel specimen...');
		Wizard.asyncJsonRequest(url, yellow.onSpecimenLoaded);
	}
	
	yellow.filterOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		var filter = new Filter();
		// Specimen filters
		filter.addContainsCriteria(ItemProperty.Name, frm.nameFilter.value);
		filter.addTextCriteria(ItemProperty.Site, frm.siteFilter.value);
		filter.addDateCriteria(ItemProperty.SamplingDate, frm.samplingDateFilter.value);
		filter.addDateCriteria(ItemProperty.ArrivalDate, frm.arrivalDateFilter.value);
		filter.addDateCriteria(ItemProperty.ConsentDate, frm.consentDateFilter.value);
		filter.addDateCriteria(ItemProperty.ExtractionDate, frm.extractionDateFilter.value);
		filter.addNumericCriteria(ItemProperty.RqsOrRin, frm.rnaqcFilter.value);
		filter.addDateCriteria(ItemProperty.LibraryDate, frm.libraryDateFilter.value);
		filter.addDateCriteria(ItemProperty.RawDate, frm.rawBioAssayDateFilter.value);
		filter.addDateCriteria(ItemProperty.PilotReportDate, frm.pilotReportDateFilter.value);
		filter.addDateCriteria(ItemProperty.CompletedDate, frm.completedDateFilter.value);
		filter.addDateCriteria(ItemProperty.DownloadedDate, frm.downloadedDateFilter.value);

		var numActive = 0;
		var numCompleted = 0;
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			
			sp.include = filter.check(sp);
			if (sp.include)
			{
				if (sp.CompletedDate)
				{
					numCompleted++;
				}
				else
				{
					numActive++;
					Doc.element('index.'+sp.id).innerHTML = numActive;
				}
				var evenRow = (numActive + numCompleted) % 2 == 0;
				var tr = Doc.element('sp.'+sp.id);
				Doc.addOrRemoveClass(tr, 'oddrow', !evenRow);
				Doc.addOrRemoveClass(tr, 'evenrow', evenRow);
				Doc.show(tr);
			}
			else
			{
				Doc.hide('sp.'+sp.id);
			}
		}
		
		Doc.showHide('specimen-list', numActive+numCompleted > 0);
		Doc.showHide('no-matching-specimen', numActive+numCompleted==0);
		Doc.element('num-active').innerHTML = numActive;
		Doc.element('num-completed').innerHTML = numCompleted;
	}
	
	yellow.asFlag = function(item)
	{
		var html = '';
		if (item && item.Flag)
		{
			html += '<img class="flagged" src="'+home+'/images/flag.png" title="'+Strings.encodeTags(item.Flag)+'">';
		}
		return html;
	}
	
	yellow.asItemLink = function(item, itemType)
	{
		var link = '';
		if (item) 
		{
			link = '<span class="link linked-item" data-item-type="'+itemType+'" data-item-id="'+item.id+'"><img src="../../../images/edit.png"></span>';
		}
		return link;
	}
	
	yellow.asFileLink = function(file, icon)
	{
		var link = '';
		if (file) 
		{
			link = '<span class="link linked-file" data-file-id="'+file.id+'"><img src="'+home+'/images/'+icon+'"></span>';
		}
		return link;
	}
	
	yellow.doNotUseWarning = function(raw, rna)
	{
		var warning = '';
		if (raw && raw.DO_NOT_USE)
		{
			warning += '<img src="'+home+'/images/warning.png" title="DoNotUse-'+Strings.encodeTags(raw.DO_NOT_USE + ': ' + raw.DO_NOT_USE_COMMENT)+'">';
		}
		else if (rna && rna.DO_NOT_USE)
		{
			warning += '<img src="'+home+'/images/warning.png" title="DoNotUse-'+Strings.encodeTags(rna.DO_NOT_USE + ': ' + rna.DO_NOT_USE_COMMENT)+'">';
		}
		return warning;
	}

	yellow.checkSelected = function(filter, noneSelectedMsg)
	{
		var frm = document.forms['reggie'];
		var selected = [];
		
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			if (sp.include && frm['sp.'+sp.id] && frm['sp.'+sp.id].checked)
			{
				if (!filter || filter.check(sp))
				selected[selected.length] = sp;
			}
		}
		
		if (selected.length == 0 && noneSelectedMsg)
		{
			Forms.showNotification('check_uncheck', noneSelectedMsg, null, 'pointer-below');
		}
		
		return selected;
	}
	
	/**
		Count the number of items in the list that has a raw bioassay
		or RNA marked as DoNotUse.
	 */
	yellow.countDoNotUse = function(items)
	{
		var numDoNotUse = 0;
		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			if (item.raw && item.raw.DO_NOT_USE || item.rna && item.rna.DO_NOT_USE) numDoNotUse++;
		}
		return numDoNotUse;
	}

	/**
		Called when clicking the button and also a callback
		for for the dialog. In the second case, options
		must be specifed.
	 */
	yellow.markAsCompleted = function(event, options)
	{
		var selected = yellow.checkSelected(null, 'Select at least one specimen');
		if (selected.length == 0) return;
		
		if (!options)
		{
			var url = 'mark-completed.jsp?ID='+App.getSessionId();
			url += '&numSelected='+selected.length;
			url += '&numDoNotUse='+yellow.countDoNotUse(selected);
			Dialogs.openPopup(url, 'MarkCompleted', 450, 300);
		}
		else
		{
			var sp = [];
			for (var i = 0; i < selected.length; i++)
			{
				var tmp = {};
				tmp.id = selected[i].id;
				sp[i] = tmp;
			}
			
			var submitInfo = {};
			submitInfo.cmd = 'MarkAsCompleted';
			submitInfo.message = 'Marking ' + selected.length + ' specimen as completed';
			submitInfo.specimen = sp;
			submitInfo.options = options;
			Wizard.goRegister(submitInfo);
		}
	}
	
	/**
		Called when clicking the button and also a callback
		for downloading the ZIP file. In the second case, options
		must be specifed.
	 */
	yellow.createPilotReportZip = function(event, options)
	{
		var frm = document.forms['reggie'];
		
		// Filter out items that have a pilot report
		var filter = new Filter();
		filter.addNumericCriteria(ItemProperty.PilotReportExists, '1');
		
		var selected = yellow.checkSelected(filter, 'Select at least one specimen that has a pilot report');
		if (selected.length == 0) return;

		if (!options)
		{
			var url = 'make-zip.jsp?ID='+App.getSessionId();
			url += '&numSelected='+selected.length;
			url += '&numDoNotUse='+yellow.countDoNotUse(selected);
			Dialogs.openPopup(url, 'MakeZip', 450, 300);
		}
		else
		{
			var raw = [];
			var specimen = [];
			for (var i = 0; i < selected.length; i++)
			{
				raw[i] = selected[i].raw.id;
				specimen[i] = selected[i].id;
			}
			
			var url = '../YellowLabel.servlet?ID='+App.getSessionId();
			url += '&cmd=CreatePilotReportZip';
			url += '&rawBioAssays='+raw.join(',');
			url += '&specimen='+specimen.join(',');
			url += '&password='+encodeURIComponent(options.password);
			url += '&completedDate='+encodeURIComponent(options.completedDate);
			url += '&site='+encodeURIComponent(frm.siteFilter.value);
			location.href = url;
		}
	}
	
	yellow.createPilotReportPdf = function()
	{
		// Filter out items that have raw data
		var filter = new Filter();
		filter.addNumericCriteria(ItemProperty.RawDataExists, '1');
		
		var selected = yellow.checkSelected(filter, 'Select at least one specimen that has raw data');
		if (selected.length == 0) return;

		var raw = [];
		for (var i = 0; i < selected.length; i++)
		{
			raw[i] = { 'id': selected[i].raw.id };
		}

		var submitInfo = {};
		submitInfo.rawBioAssays = raw;
		submitInfo.useDefaultSoftware = 1;
		submitInfo.report = 'PILOT_REPORT';
		
		var url = '../Report.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateReport';
		Wizard.asyncJsonRequest(url, yellow.reportJobSubmitted, 'POST', JSON.stringify(submitInfo));
	}

	/**
		Called when clicking the button and also a callback
		for starting the publish job. In the second case, options
		must be specifed.
	 */
	yellow.publishForDelivery = function(event, options)
	{
		// Filter out items that have a pilot report
		var filter = new Filter();
		filter.addNumericCriteria(ItemProperty.PilotReportExists, '1');
		
		var selected = yellow.checkSelected(filter, 'Select at least one specimen that has a pilot report');
		if (selected.length == 0) return;
	
		if (!options)
		{
			var url = 'publish-for-delivery.jsp?ID='+App.getSessionId();
			url += '&numSelected='+selected.length;
			url += '&numDoNotUse='+yellow.countDoNotUse(selected);
			Dialogs.openPopup(url, 'PublishForDelivery', 450, 300);
		}
		else
		{
			var raw = [];
			var specimen = [];
			for (var i = 0; i < selected.length; i++)
			{
				raw[i] = {'id': selected[i].raw.id };
				specimen[i] = {'id': selected[i].id };
			}
			
			var submitInfo = {};
			submitInfo.cmd = 'PublishForDelivery';
			submitInfo.rawBioAssays = raw;
			submitInfo.specimen = specimen;
			submitInfo.options = options;
			
			Wizard.goRegister(submitInfo);
		}
	}
	
	yellow.reportJobSubmitted = function(response)
	{
		if (response.job)
		{
			Items.viewItem('JOB', response.job.id);
			location.reload(true);
		}	
	}
	
	yellow.submit = function(event)
	{
		var submitInfo = event.detail;
		var url = '../YellowLabel.servlet?ID='+App.getSessionId();
		url += '&cmd='+submitInfo.cmd;
		Wizard.showLoadingAnimation(submitInfo.message || 'Registering YellowLabel specimen...');
		Wizard.asyncJsonRequest(url, yellow.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	yellow.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return yellow;
}();

Doc.onLoad(YellowLabel.initPage);


/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(item)' method
	to evaluate all criterias against a single item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given item property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a numeric criteria comparing the given item property to
	a static value (f2). The 'f2' value may start with either '<' or '>'
	to compare with less than/greater than operators, otherwise
	it will use the 'approximately equals' operator.
*/
Filter.prototype.addNumericCriteria = function(itemProp, f2, factor)
{
	if (f2 != '')
	{
		var cmp;
		var limit;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			limit = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			limit = null;
		}
		else
		{
			var firstChar = f2.charAt(0);
			if (!firstChar.match(/[\d\-]/)) f2 = f2.substring(1);

			cmp = Comparator.get(firstChar, Comparator.approx);
			limit = parseFloat(f2);
			if (factor) limit *= factor;
		}
		

		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), limit) };
	}
}

/**
	Adds a temporal criteria for comparing dates. The 'f2' value
	may start with either '<' or '>' to compare with earlier/later
	than operators. In all cases, the 'f2' value is stripped from '-'
	characters.
 */
Filter.prototype.addDateCriteria = function(itemProp, f2)
{
	if (f2 != '')
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			var firstChar = f2.charAt(0);
			if (firstChar == '<' || firstChar == '>') f2 = f2.substring(1);
			f2 = f2.replace(/\-/g, '');
			cmp = Comparator.get(firstChar, Comparator.startsWith);
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
	
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(item)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](item)) return false;
	}
	return true;
}

/**
	Functions for extracting various properties from
	specimen items. Each method accepts a single specimen as a parameter
	and return a value from the specimen item. The methods
	are used as parameters to Filter.add*Criteria() methods.
*/
var ItemProperty = function()
{
	var p = {};
	
	p.Name = function(item)
	{
		return item.name;
	}
	
	// Get the item.site.prefix value
	p.Site = function(item)
	{
		return item.site.prefix;
	}

	p.ArrivalDate = function(item)
	{
		return item.ArrivalDate;
	}

	p.SamplingDate = function(item)
	{
		return item.SamplingDate;
	}
	
	p.ConsentDate = function(item)
	{
		return item.ConsentDate;
	}
	
	p.ExtractionDate = function(item)
	{
		return item.rna ? item.rna.QiacubeDate : null;
	}
	
	p.RqsOrRin = function(item)
	{
		return item.rnaqc ? (item.rnaqc.RQS || item.rnaqc.RIN) : null;
	}
	
	p.LibraryDate = function(item)
	{
		return item.library ? item.library.creationDate : null;
	}
	
	p.RawDate = function(item)
	{
		return item.raw ? item.raw.creationDate : null;
	}
	
	p.RawDataExists = function(item)
	{
		return item.raw ? 1 : 0;
	}
	
	p.PilotReportExists = function(item)
	{
		return item.pilotReport ? 1 : 0;
	}

	p.PilotReportDate = function(item)
	{
		return item.pilotReport ? item.pilotReport.lastUpdatedDate : null;
	}
	
	p.CompletedDate = function(item)
	{
		return item.CompletedDate;
	}
	
	p.DownloadedDate = function(item)
	{
		return item.pilotReport ? item.pilotReport.downloadedDate : null;
	}

	return p;
}();

/**
	Comparator functions for comparing two values.
*/
var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 <= v2;
	}
	
	return cmp;
}();

